@@ -35,6 +35,11 @@
 #include "bebob/edirol/edirol_fa101.h"
 #include "bebob/edirol/edirol_fa66.h"
 #include "bebob/esi/quatafire610.h"
+#include "bebob/yamaha/yamaha_avdevice.h"
+#include "bebob/maudio/normal_avdevice.h"
+#include "bebob/maudio/special_avdevice.h"
+#include "bebob/presonus/firebox_avdevice.h"
+#include "bebob/presonus/inspire1394_avdevice.h"
 
 #include "libieee1394/configrom.h"
 #include "libieee1394/ieee1394service.h"
@@ -59,24 +64,32 @@ using namespace AVC;
 
 namespace BeBoB {
 
-AvDevice::AvDevice( DeviceManager& d, std::auto_ptr< ConfigRom >( configRom ) )
-    : GenericAVC::AvDevice( d, configRom )
+Device::Device( DeviceManager& d, std::auto_ptr< ConfigRom >( configRom ) )
+    : GenericAVC::Device( d, configRom )
     , m_last_discovery_config_id ( 0xFFFFFFFFFFFFFFFFLLU )
     , m_Mixer ( 0 )
 {
-    debugOutput( DEBUG_LEVEL_VERBOSE, "Created BeBoB::AvDevice (NodeID %d)\n",
+    debugOutput( DEBUG_LEVEL_VERBOSE, "Created BeBoB::Device (NodeID %d)\n",
                  getConfigRom().getNodeId() );
 }
 
-AvDevice::~AvDevice()
+Device::~Device()
 {
     destroyMixer();
 }
 
 bool
-AvDevice::probe( Util::Configuration& c, ConfigRom& configRom, bool generic )
+Device::probe( Util::Configuration& c, ConfigRom& configRom, bool generic )
 {
+    unsigned int vendorId = configRom.getNodeVendorId();
+    unsigned int modelId = configRom.getModelId();
+
     if(generic) {
+        /* M-Audio Special Devices don't support followed commands */
+        if ((vendorId == FW_VENDORID_MAUDIO) &&
+            ((modelId == 0x00010071) || (modelId == 0x00010091)))
+            return true;
+
         // try a bebob-specific command to check for the firmware
         ExtendedPlugInfoCmd extPlugInfoCmd( configRom.get1394Service() );
         UnitPlugAddress unitPlugAddress( UnitPlugAddress::ePT_PCR,
@@ -111,16 +124,13 @@ AvDevice::probe( Util::Configuration& c, ConfigRom& configRom, bool generic )
         return false;
     } else {
         // check if device is in supported devices list
-        unsigned int vendorId = configRom.getNodeVendorId();
-        unsigned int modelId = configRom.getModelId();
-
         Util::Configuration::VendorModelEntry vme = c.findDeviceVME( vendorId, modelId );
         return c.isValid(vme) && vme.driver == Util::Configuration::eD_BeBoB;
     }
 }
 
 FFADODevice *
-AvDevice::createDevice(DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
+Device::createDevice(DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
 {
     unsigned int vendorId = configRom->getNodeVendorId();
     unsigned int modelId = configRom->getModelId();
@@ -137,7 +147,7 @@ AvDevice::createDevice(DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
                 case 0x00010049:
                     return new Edirol::EdirolFa66Device(d, configRom);
                 default:
-                    return new AvDevice(d, configRom);
+                    return new Device(d, configRom);
             }
         case FW_VENDORID_ESI:
             if (modelId == 0x00010064) {
@@ -149,7 +159,7 @@ AvDevice::createDevice(DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
                 case 0x00000003:
                     return new Terratec::Phase88Device(d, configRom);
                 default: // return a plain BeBoB device
-                    return new AvDevice(d, configRom);
+                    return new Device(d, configRom);
             }
         case FW_VENDORID_FOCUSRITE:
             switch(modelId) {
@@ -159,10 +169,40 @@ AvDevice::createDevice(DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
                 case 0x00000000:
                     return new Focusrite::SaffireDevice(d, configRom);
                 default: // return a plain BeBoB device
-                    return new AvDevice(d, configRom);
+                    return new Device(d, configRom);
            }
+        case FW_VENDORID_YAMAHA:
+        switch (modelId) {
+            case 0x0010000b:
+            case 0x0010000c:
+                return new Yamaha::GoDevice(d, configRom);
+            default: // return a plain BeBoB device
+                return new Device(d, configRom);
+        }
+        case FW_VENDORID_MAUDIO:
+        switch (modelId) {
+        case 0x0000000a: // Ozonic
+        case 0x00010046: // fw410
+        case 0x00010060: // Audiophile
+        case 0x00010062: // Solo
+            return new MAudio::Normal::Device(d, configRom, modelId);
+        case 0x00010071: // Firewire 1814
+        case 0x00010091: // ProjectMix I/O
+            return new MAudio::Special::Device(d, configRom);
         default:
-            return new AvDevice(d, configRom);
+            return new Device(d, configRom);
+        }
+        case FW_VENDORID_PRESONUS:
+        switch (modelId) {
+        case 0x00010000:
+            return new Presonus::Firebox::Device(d, configRom);
+        case 0x00010001:
+            return new Presonus::Inspire1394::Device(d, configRom);
+        default:
+            return new Device(d, configRom);
+        }
+        default:
+            return new Device(d, configRom);
     }
     return NULL;
 }
@@ -171,7 +211,7 @@ AvDevice::createDevice(DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
     { if(supportsSamplingFrequency(x)) \
       v.push_back(x); }
 bool
-AvDevice::discover()
+Device::discover()
 {
     unsigned int vendorId = getConfigRom().getNodeVendorId();
     unsigned int modelId = getConfigRom().getModelId();
@@ -213,7 +253,7 @@ AvDevice::discover()
 }
 
 bool
-AvDevice::buildMixer()
+Device::buildMixer()
 {
     debugOutput(DEBUG_LEVEL_VERBOSE, "Building a generic BeBoB mixer...\n");
     // create a Mixer
@@ -233,14 +273,14 @@ AvDevice::buildMixer()
 }
 
 bool
-AvDevice::destroyMixer()
+Device::destroyMixer()
 {
     delete m_Mixer;
     return true;
 }
 
 bool
-AvDevice::setSelectorFBValue(int id, int value) {
+Device::setSelectorFBValue(int id, int value) {
     FunctionBlockCmd fbCmd( get1394Service(),
                             FunctionBlockCmd::eFBT_Selector,
                             id,
@@ -269,7 +309,7 @@ AvDevice::setSelectorFBValue(int id, int value) {
 }
 
 int
-AvDevice::getSelectorFBValue(int id) {
+Device::getSelectorFBValue(int id) {
 
     FunctionBlockCmd fbCmd( get1394Service(),
                             FunctionBlockCmd::eFBT_Selector,
@@ -299,7 +339,7 @@ AvDevice::getSelectorFBValue(int id) {
 }
 
 bool
-AvDevice::setFeatureFBVolumeCurrent(int id, int channel, int v) {
+Device::setFeatureFBVolumeCurrent(int id, int channel, int v) {
 
     FunctionBlockCmd fbCmd( get1394Service(),
                             FunctionBlockCmd::eFBT_Feature,
@@ -333,7 +373,7 @@ AvDevice::setFeatureFBVolumeCurrent(int id, int channel, int v) {
 }
 
 int
-AvDevice::getFeatureFBVolumeValue(int id, int channel, FunctionBlockCmd::EControlAttribute controlAttribute) 
+Device::getFeatureFBVolumeValue(int id, int channel, FunctionBlockCmd::EControlAttribute controlAttribute) 
 {
     FunctionBlockCmd fbCmd( get1394Service(),
                             FunctionBlockCmd::eFBT_Feature,
@@ -369,25 +409,25 @@ AvDevice::getFeatureFBVolumeValue(int id, int channel, FunctionBlockCmd::EContro
 }
 
 int 
-AvDevice::getFeatureFBVolumeMinimum(int id, int channel)
+Device::getFeatureFBVolumeMinimum(int id, int channel)
 {
     return getFeatureFBVolumeValue(id, channel, AVC::FunctionBlockCmd::eCA_Minimum);
 }
 
 int 
-AvDevice::getFeatureFBVolumeMaximum(int id, int channel)
+Device::getFeatureFBVolumeMaximum(int id, int channel)
 {
     return getFeatureFBVolumeValue(id, channel, AVC::FunctionBlockCmd::eCA_Maximum);
 }
 
 int 
-AvDevice::getFeatureFBVolumeCurrent(int id, int channel)
+Device::getFeatureFBVolumeCurrent(int id, int channel)
 {
     return getFeatureFBVolumeValue(id, channel, AVC::FunctionBlockCmd::eCA_Current);   
 }
 
 bool
-AvDevice::setFeatureFBLRBalanceCurrent(int id, int channel, int v) {
+Device::setFeatureFBLRBalanceCurrent(int id, int channel, int v) {
 
     FunctionBlockCmd fbCmd( get1394Service(),
                             FunctionBlockCmd::eFBT_Feature,
@@ -421,7 +461,7 @@ AvDevice::setFeatureFBLRBalanceCurrent(int id, int channel, int v) {
 }
 
 int
-AvDevice::getFeatureFBLRBalanceValue(int id, int channel, FunctionBlockCmd::EControlAttribute controlAttribute) 
+Device::getFeatureFBLRBalanceValue(int id, int channel, FunctionBlockCmd::EControlAttribute controlAttribute) 
 {
     FunctionBlockCmd fbCmd( get1394Service(),
                             FunctionBlockCmd::eFBT_Feature,
@@ -457,41 +497,113 @@ AvDevice::getFeatureFBLRBalanceValue(int id, int channel, FunctionBlockCmd::ECon
 }
 
 int 
-AvDevice::getFeatureFBLRBalanceMinimum(int id, int channel)
+Device::getFeatureFBLRBalanceMinimum(int id, int channel)
 {
     return getFeatureFBLRBalanceValue(id, channel, AVC::FunctionBlockCmd::eCA_Minimum);
 }
 
 int 
-AvDevice::getFeatureFBLRBalanceMaximum(int id, int channel)
+Device::getFeatureFBLRBalanceMaximum(int id, int channel)
 {
     return getFeatureFBLRBalanceValue(id, channel, AVC::FunctionBlockCmd::eCA_Maximum);
 }
 
 int 
-AvDevice::getFeatureFBLRBalanceCurrent(int id, int channel)
+Device::getFeatureFBLRBalanceCurrent(int id, int channel)
 {
     return getFeatureFBLRBalanceValue(id, channel, AVC::FunctionBlockCmd::eCA_Current);   
 }
 
+bool
+Device::setProcessingFBMixerSingleCurrent(int id, int iPlugNum,
+                                    int iAChNum, int oAChNum,
+                                    int setting)
+{
+	AVC::FunctionBlockCmd fbCmd(get1394Service(),
+                           AVC::FunctionBlockCmd::eFBT_Processing,
+                           id,
+                           AVC::FunctionBlockCmd::eCA_Current);
+    fbCmd.setNodeId(getNodeId());
+    fbCmd.setSubunitId(0x00);
+    fbCmd.setCommandType(AVCCommand::eCT_Control);
+    fbCmd.setVerboseLevel( getDebugLevel() );
+
+    AVC::FunctionBlockProcessing *fbp = fbCmd.m_pFBProcessing;
+    fbp->m_selectorLength = 0x04;
+    fbp->m_fbInputPlugNumber = iPlugNum;
+    fbp->m_inputAudioChannelNumber = iAChNum;
+    fbp->m_outputAudioChannelNumber = oAChNum;
+
+    // mixer object is not generated automatically
+    fbp->m_pMixer = new AVC::FunctionBlockProcessingMixer;
+    fbp->m_pMixer->m_mixerSetting = setting;
+
+    if ( !fbCmd.fire() ) {
+        debugError( "cmd failed\n" );
+        return false;
+    }
+
+    if((fbCmd.getResponse() != AVCCommand::eR_Accepted)) {
+        debugWarning("fbCmd.getResponse() != AVCCommand::eR_Accepted\n");
+    }
+
+    return (fbCmd.getResponse() == AVCCommand::eR_Accepted);
+}
+
+int
+Device::getProcessingFBMixerSingleCurrent(int id, int iPlugNum,
+                                    int iAChNum, int oAChNum)
+{
+	AVC::FunctionBlockCmd fbCmd(get1394Service(),
+                           AVC::FunctionBlockCmd::eFBT_Processing,
+                           id,
+                           AVC::FunctionBlockCmd::eCA_Current);
+    fbCmd.setNodeId(getNodeId());
+    fbCmd.setSubunitId(0x00);
+    fbCmd.setCommandType(AVCCommand::eCT_Status);
+    fbCmd.setVerboseLevel( getDebugLevel() );
+
+    AVC::FunctionBlockProcessing *fbp = fbCmd.m_pFBProcessing;
+    fbp->m_selectorLength = 0x04;
+    fbp->m_fbInputPlugNumber = iPlugNum;
+    fbp->m_inputAudioChannelNumber = iAChNum;
+    fbp->m_outputAudioChannelNumber = oAChNum;
+
+    // mixer object is not generated automatically
+    fbp->m_pMixer = new AVC::FunctionBlockProcessingMixer;
+
+    if ( !fbCmd.fire() ) {
+        debugError( "cmd failed\n" );
+        return 0;
+    }
+
+    if( (fbCmd.getResponse() != AVCCommand::eR_Implemented) ) {
+        debugWarning("fbCmd.getResponse() != AVCCommand::eR_Implemented\n");
+    }
+
+    int16_t setting = (int16_t)(fbp->m_pMixer->m_mixerSetting);
+
+    return setting;
+}
+
 void
-AvDevice::showDevice()
+Device::showDevice()
 {
     debugOutput(DEBUG_LEVEL_NORMAL, "Device is a BeBoB device\n");
-    GenericAVC::AvDevice::showDevice();
+    GenericAVC::Device::showDevice();
     flushDebugOutput();
 }
 
 void
-AvDevice::setVerboseLevel(int l)
+Device::setVerboseLevel(int l)
 {
     if (m_Mixer) m_Mixer->setVerboseLevel( l );
-    GenericAVC::AvDevice::setVerboseLevel( l );
+    GenericAVC::Device::setVerboseLevel( l );
     debugOutput( DEBUG_LEVEL_VERBOSE, "Setting verbose level to %d...\n", l );
 }
 
 AVC::Subunit*
-AvDevice::createSubunit(AVC::Unit& unit,
+Device::createSubunit(AVC::Unit& unit,
                         AVC::ESubunitType type,
                         AVC::subunit_t id )
 {
@@ -513,7 +625,7 @@ AvDevice::createSubunit(AVC::Unit& unit,
 
 
 AVC::Plug *
-AvDevice::createPlug( AVC::Unit* unit,
+Device::createPlug( AVC::Unit* unit,
                       AVC::Subunit* subunit,
                       AVC::function_block_type_t functionBlockType,
                       AVC::function_block_type_t functionBlockId,
@@ -536,7 +648,7 @@ AvDevice::createPlug( AVC::Unit* unit,
 }
 
 bool
-AvDevice::propagatePlugInfo() {
+Device::propagatePlugInfo() {
     // we don't have to propagate since we discover things
     // another way
     debugOutput(DEBUG_LEVEL_VERBOSE, "Skip plug info propagation\n");
@@ -544,7 +656,7 @@ AvDevice::propagatePlugInfo() {
 }
 
 uint8_t
-AvDevice::getConfigurationIdSampleRate()
+Device::getConfigurationIdSampleRate()
 {
     ExtendedStreamFormatCmd extStreamFormatCmd( get1394Service() );
     UnitPlugAddress unitPlugAddress( UnitPlugAddress::ePT_PCR, 0 );
@@ -577,7 +689,7 @@ AvDevice::getConfigurationIdSampleRate()
 }
 
 uint8_t
-AvDevice::getConfigurationIdNumberOfChannel( PlugAddress::EPlugDirection ePlugDirection )
+Device::getConfigurationIdNumberOfChannel( PlugAddress::EPlugDirection ePlugDirection )
 {
     ExtendedPlugInfoCmd extPlugInfoCmd( get1394Service() );
     UnitPlugAddress unitPlugAddress( UnitPlugAddress::ePT_PCR,
@@ -612,7 +724,7 @@ AvDevice::getConfigurationIdNumberOfChannel( PlugAddress::EPlugDirection ePlugDi
 }
 
 uint16_t
-AvDevice::getConfigurationIdSyncMode()
+Device::getConfigurationIdSyncMode()
 {
     SignalSourceCmd signalSourceCmd( get1394Service() );
     SignalUnitAddress signalUnitAddr;
@@ -658,7 +770,7 @@ AvDevice::getConfigurationIdSyncMode()
 }
 
 bool
-AvDevice::needsRediscovery()
+Device::needsRediscovery()
 {
     // require rediscovery if the config id differs from the one saved
     // in the previous discovery
@@ -666,7 +778,7 @@ AvDevice::needsRediscovery()
 }
 
 uint64_t
-AvDevice::getConfigurationId()
+Device::getConfigurationId()
 {
     // create a unique configuration id.
     uint64_t id = 0;
@@ -678,25 +790,25 @@ AvDevice::getConfigurationId()
 }
 
 bool
-AvDevice::serialize( std::string basePath,
+Device::serialize( std::string basePath,
                      Util::IOSerialize& ser ) const
 {
     bool result;
-    result  = GenericAVC::AvDevice::serialize( basePath, ser );
+    result  = GenericAVC::Device::serialize( basePath, ser );
     return result;
 }
 
 bool
-AvDevice::deserialize( std::string basePath,
+Device::deserialize( std::string basePath,
                        Util::IODeserialize& deser )
 {
     bool result;
-    result  = GenericAVC::AvDevice::deserialize( basePath, deser );
+    result  = GenericAVC::Device::deserialize( basePath, deser );
     return result;
 }
 
 std::string
-AvDevice::getCachePath()
+Device::getCachePath()
 {
     std::string cachePath;
     char* pCachePath;
@@ -709,7 +821,7 @@ AvDevice::getCachePath()
 
     if ( asprintf( &pCachePath, "%s/cache/",  path.c_str() ) < 0 ) {
         debugError( "Could not create path string for cache pool (trying '/var/cache/libffado' instead)\n" );
-        cachePath == "/var/cache/libffado/";
+        cachePath = "/var/cache/libffado/";
     } else {
         cachePath = pCachePath;
         free( pCachePath );
@@ -718,7 +830,7 @@ AvDevice::getCachePath()
 }
 
 bool
-AvDevice::loadFromCache()
+Device::loadFromCache()
 {
     std::string sDevicePath = getCachePath() + getConfigRom().getGuidString();
 
@@ -766,7 +878,7 @@ AvDevice::loadFromCache()
 }
 
 bool
-AvDevice::saveCache()
+Device::saveCache()
 {
     // the path looks like this:
     // PATH_TO_CACHE + GUID + CONFIGURATION_ID
@@ -799,7 +911,7 @@ AvDevice::saveCache()
 
     // come up with an unique file name for the current settings
     char* configId;
-    asprintf(&configId, "%016"PRIx64"", BeBoB::AvDevice::getConfigurationId() );
+    asprintf(&configId, "%016"PRIx64"", BeBoB::Device::getConfigurationId() );
     if ( !configId ) {
         debugError( "Could not create id string\n" );
         return false;
