@@ -22,6 +22,7 @@
  *
  */
 
+#include "config.h"
 #include "fbtypes.h"
 
 #include "devicemanager.h"
@@ -51,10 +52,31 @@
     #include "fireworks/fireworks_device.h"
 #endif
 
+#ifdef ENABLE_OXFORD
+    #include "oxford/oxford_device.h"
+#endif
+
+#ifdef ENABLE_BOUNCE
+#include "bounce/bounce_avdevice.h"
+#include "bounce/bounce_slave_avdevice.h"
+#endif
+
 #ifdef ENABLE_MOTU
 #include "motu/motu_avdevice.h"
 #endif
 
+#ifdef ENABLE_RME
+#include "rme/rme_avdevice.h"
+#endif
+
+#ifdef ENABLE_DICE
+#include "dice/dice_avdevice.h"
+#endif
+
+#ifdef ENABLE_METRIC_HALO
+#include "metrichalo/mh_avdevice.h"
+#endif
+
 #include <iostream>
 #include <sstream>
 
@@ -75,8 +97,8 @@ DeviceManager::DeviceManager()
     , m_thread_realtime( false )
     , m_thread_priority( 0 )
 {
-    addOption(Util::OptionContainer::Option("slaveMode",false));
-    addOption(Util::OptionContainer::Option("snoopMode",false));
+    addOption(Util::OptionContainer::Option("slaveMode", false));
+    addOption(Util::OptionContainer::Option("snoopMode", false));
 }
 
 DeviceManager::~DeviceManager()
@@ -93,7 +115,7 @@ DeviceManager::~DeviceManager()
           ++it )
     {
         if (!deleteElement(*it)) {
-            debugWarning("failed to remove AvDevice from Control::Container\n");
+            debugWarning("failed to remove Device from Control::Container\n");
         }
         delete *it;
     }
@@ -412,7 +434,7 @@ DeviceManager::discover( bool useCache, bool rediscover )
 
                 // the device has disappeared, remove it from the control tree
                 if (!deleteElement(*it)) {
-                    debugWarning("failed to remove AvDevice from Control::Container\n");
+                    debugWarning("failed to remove Device from Control::Container\n");
                 }
                 // delete the device
                 // FIXME: this will mess up the any code that waits for bus resets to
@@ -428,7 +450,7 @@ DeviceManager::discover( bool useCache, bool rediscover )
             ++it )
         {
             if (!deleteElement(*it)) {
-                debugWarning("failed to remove AvDevice from Control::Container\n");
+                debugWarning("failed to remove Device from Control::Container\n");
             }
             delete *it;
         }
@@ -515,7 +537,7 @@ DeviceManager::discover( bool useCache, bool rediscover )
             ++it2 )
         {
             if (!deleteElement(*it2)) {
-                debugWarning("failed to remove AvDevice from Control::Container\n");
+                debugWarning("failed to remove Device from Control::Container\n");
             }
             delete *it2;
         }
@@ -625,7 +647,7 @@ DeviceManager::discover( bool useCache, bool rediscover )
                     m_avDevices.push_back( avDevice );
 
                     if (!addElement(avDevice)) {
-                        debugWarning("failed to add AvDevice to Control::Container\n");
+                        debugWarning("failed to add Device to Control::Container\n");
                     }
 
                     debugOutput( DEBUG_LEVEL_NORMAL, "discovery of node %d on port %d done...\n", nodeId, portService->getPort() );
@@ -678,16 +700,6 @@ DeviceManager::discover( bool useCache, bool rediscover )
             m_avDevices = sorted;
         }
 
-        // set device id's
-        i = 0;
-        for ( FFADODeviceVectorIterator it = m_avDevices.begin();
-            it != m_avDevices.end();
-            ++it )
-        {
-            if ( !(*it)->setId( i++ ) ) {
-                debugError( "setting Id failed\n" );
-            }
-        }
         showDeviceInfo();
 
     } else { // slave mode
@@ -719,7 +731,7 @@ DeviceManager::discover( bool useCache, bool rediscover )
             ++it )
         {
             if (!deleteElement(*it)) {
-                debugWarning("failed to remove AvDevice from Control::Container\n");
+                debugWarning("failed to remove Device from Control::Container\n");
             }
             delete *it;
         }
@@ -741,9 +753,6 @@ DeviceManager::discover( bool useCache, bool rediscover )
                 return false;
             }
 
-            if ( !avDevice->setId( m_avDevices.size() ) ) {
-                debugError( "setting Id failed\n" );
-            }
             if ( getDebugLevel() >= DEBUG_LEVEL_VERBOSE ) {
                 avDevice->showDevice();
             }
@@ -843,6 +852,10 @@ DeviceManager::startStreamingOnDevice(FFADODevice *device)
     int j=0;
     bool all_streams_started = true;
     bool device_start_failed = false;
+
+    if (device->resetForStreaming() == false)
+        return false;
+
     for(j=0; j < device->getStreamCount(); j++) {
         debugOutput(DEBUG_LEVEL_VERBOSE,"Starting stream %d of device %p\n", j, device);
         // start the stream
@@ -990,8 +1003,21 @@ DeviceManager::waitForPeriod() {
     }
 }
 
+bool
+DeviceManager::setPeriodSize(unsigned int period) {
+    // Useful for cases where only the period size needs adjusting
+    if (!m_processorManager->streamingParamsOk(period, -1, -1)) {
+        return false;
+    }
+    m_processorManager->setPeriodSize(period);
+    return true;
+}
+
 bool
 DeviceManager::setStreamingParams(unsigned int period, unsigned int rate, unsigned int nb_buffers) {
+    if (!m_processorManager->streamingParamsOk(period, rate, nb_buffers)) {
+        return false;
+    }
     m_processorManager->setPeriodSize(period);
     m_processorManager->setNominalRate(rate);
     m_processorManager->setNbBuffers(nb_buffers);
@@ -1004,8 +1030,8 @@ DeviceManager::getDriverForDeviceDo( ConfigRom *configRom,
 {
 #ifdef ENABLE_BEBOB
     debugOutput( DEBUG_LEVEL_VERBOSE, "Trying BeBoB...\n" );
-    if ( BeBoB::AvDevice::probe( getConfiguration(), *configRom, generic ) ) {
-        return BeBoB::AvDevice::createDevice( *this, std::auto_ptr<ConfigRom>( configRom ) );
+    if ( BeBoB::Device::probe( getConfiguration(), *configRom, generic ) ) {
+        return BeBoB::Device::createDevice( *this, std::auto_ptr<ConfigRom>( configRom ) );
     }
 #endif
 
@@ -1016,11 +1042,18 @@ DeviceManager::getDriverForDeviceDo( ConfigRom *configRom,
     }
 #endif
 
+#ifdef ENABLE_OXFORD
+    debugOutput( DEBUG_LEVEL_VERBOSE, "Trying Oxford FW90x...\n" );
+    if ( Oxford::Device::probe( getConfiguration(), *configRom, generic ) ) {
+        return Oxford::Device::createDevice( *this, std::auto_ptr<ConfigRom>( configRom ) );
+    }
+#endif
+
 // we want to try the non-generic AV/C platforms before trying the generic ones
 #ifdef ENABLE_GENERICAVC
     debugOutput( DEBUG_LEVEL_VERBOSE, "Trying Generic AV/C...\n" );
-    if ( GenericAVC::AvDevice::probe( getConfiguration(), *configRom, generic ) ) {
-        return GenericAVC::AvDevice::createDevice( *this, std::auto_ptr<ConfigRom>( configRom ) );
+    if ( GenericAVC::Device::probe( getConfiguration(), *configRom, generic ) ) {
+        return GenericAVC::Device::createDevice( *this, std::auto_ptr<ConfigRom>( configRom ) );
     }
 #endif
 
@@ -1031,6 +1064,34 @@ DeviceManager::getDriverForDeviceDo( ConfigRom *configRom,
     }
 #endif
 
+#ifdef ENABLE_DICE
+    debugOutput( DEBUG_LEVEL_VERBOSE, "Trying Dice...\n" );
+    if ( Dice::Device::probe( getConfiguration(), *configRom, generic ) ) {
+        return Dice::Device::createDevice( *this, std::auto_ptr<ConfigRom>( configRom ) );
+    }
+#endif
+
+#ifdef ENABLE_METRIC_HALO
+    debugOutput( DEBUG_LEVEL_VERBOSE, "Trying Metric Halo...\n" );
+    if ( MetricHalo::Device::probe( getConfiguration(), *configRom, generic ) ) {
+        return MetricHalo::Device::createDevice( *this, std::auto_ptr<ConfigRom>( configRom ) );
+    }
+#endif
+
+#ifdef ENABLE_RME
+    debugOutput( DEBUG_LEVEL_VERBOSE, "Trying RME...\n" );
+    if ( Rme::Device::probe( getConfiguration(), *configRom, generic ) ) {
+        return Rme::Device::createDevice( *this, std::auto_ptr<ConfigRom>( configRom ) );
+    }
+#endif
+
+#ifdef ENABLE_BOUNCE
+    debugOutput( DEBUG_LEVEL_VERBOSE, "Trying Bounce...\n" );
+    if ( Bounce::Device::probe( getConfiguration(), *configRom, generic ) ) {
+        return Bounce::Device::createDevice( *this, std::auto_ptr<ConfigRom>( configRom ) );
+    }
+#endif
+
     return NULL;
 }
 
@@ -1060,6 +1121,11 @@ DeviceManager::getDriverForDevice( ConfigRom *configRom,
 FFADODevice*
 DeviceManager::getSlaveDriver( std::auto_ptr<ConfigRom>( configRom ) )
 {
+#ifdef ENABLE_BOUNCE
+    if ( Bounce::SlaveDevice::probe( getConfiguration(), *configRom, false ) ) {
+        return Bounce::SlaveDevice::createDevice(  *this, std::auto_ptr<ConfigRom>( configRom ) );
+    }
+#endif
     return NULL;
 }
 
@@ -1148,6 +1214,14 @@ DeviceManager::getSyncSource() {
                     "Could not retrieve slaveMode parameter, defauling to false\n");
     }
     return device->getStreamProcessorByIndex(0);
+/*
+    #warning TEST CODE FOR BOUNCE DEVICE !!
+    // this makes the bounce slave use the xmit SP as sync source
+    if (slaveMode) {
+        return device->getStreamProcessorByIndex(1);
+    } else {
+        return device->getStreamProcessorByIndex(0);
+    }*/
 }
 
 bool
