@@ -37,6 +37,7 @@
 #include <stdint.h>
 #include <iostream>
 #include <sstream>
+#include <cstdio>
 
 #include <string>
 #include <cstring>
@@ -47,11 +48,13 @@
 #include "devicemanager.h"
 
 #include "focusrite/saffire_pro40.h"
+#include "focusrite/saffire_pro26.h"
 #include "focusrite/saffire_pro24.h"
 #include "focusrite/saffire_pro14.h"
 #include "maudio/profire_2626.h"
 #include "presonus/firestudio_tube.h"
 #include "presonus/firestudio_project.h"
+#include "presonus/firestudio_mobile.h"
 
 using namespace std;
 
@@ -137,6 +140,8 @@ Device::createDevice( DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
                     return new Focusrite::SaffirePro24(d, configRom);
                 case 0x00000009:
                     return new Focusrite::SaffirePro14(d, configRom);
+                case 0x00000012:
+                    return new Focusrite::SaffirePro26(d, configRom);
                 default: // return a plain Dice device
                     return new Device(d, configRom);
             }
@@ -153,6 +158,8 @@ Device::createDevice( DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
                     return new Presonus::FirestudioTube(d, configRom);
                 case 0x0000000b:
                     return new Presonus::FirestudioProject(d, configRom);
+                case 0x00000011:
+                    return new Presonus::FirestudioMobile(d, configRom);
                 default: // return a plain Dice device
                     return new Device(d, configRom);
             }
@@ -1185,7 +1192,11 @@ Device::lock() {
         }
     
         if (result != DICE_OWNER_NO_OWNER && result != swap_value) {
-            debugWarning("Could not register ourselves as device owner, unexpected register value: 0x%016"PRIX64"\n", result);
+            debugWarning("Unexpected GLOBAL_OWNER register value: 0x%016"PRIX64"\n", result);
+            fprintf(stderr, "Could not register ourselves as owner of %s.\n", getNickname().c_str());
+            fprintf(stderr, "If the snd-dice kernel driver is present, "
+                            "either use the device via ALSA instead of FFADO, "
+                            "or unload snd-dice before using FFADO.\n");
             return false;
         }
     
@@ -1667,9 +1678,11 @@ Device::initIoFunctions() {
     }
     m_rx_size*=4;
 
-    // FIXME: verify this and clean it up.
+    // FIXME: verify this and clean it up. Maybe check the number of channels
+    // and ignore receivers with zero channels?
     /* special case for Alesis io14, which announces two receive transmitters,
-     * but only has one. Same is true for Alesis Multimix16.
+     * but only has one. Same is true for Alesis Multimix16 and Focusrite
+     * Saffire PRO 26.
      */
     if (FW_VENDORID_ALESIS == getConfigRom().getNodeVendorId()) {
         switch (getConfigRom().getModelId()) {
@@ -1679,6 +1692,13 @@ Device::initIoFunctions() {
                 break;
         }
     }
+    if (FW_VENDORID_FOCUSRITE == getConfigRom().getNodeVendorId()) {
+        switch (getConfigRom().getModelId()) {
+            case 0x00000012:
+                m_nb_rx = 1;
+                break;
+        }
+    }
 
 #if USE_OLD_DEFENSIVE_STREAMING_PROTECTION
     // FIXME: after a crash, the device might still be streaming. We
