@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2005-2008 by Daniel Wagner
  * Copyright (C) 2005-2008 by Pieter Palmers
+ * Copyright (C) 2012 by Jonathan Woithe
  *
  * This file is part of FFADO
  * FFADO = Free Firewire (pro-)audio drivers for linux
@@ -23,17 +24,20 @@
  */
 
 #include "config.h"
+
 #include "ieee1394service.h"
 #include "cycletimer.h"
 #include "IsoHandlerManager.h"
 #include "CycleTimerHelper.h"
 
+#include <unistd.h>
 #include <libraw1394/csr.h>
 #include <libiec61883/iec61883.h>
 
 #include "libutil/SystemTimeSource.h"
 #include "libutil/Watchdog.h"
 #include "libutil/PosixMutex.h"
+#include "libutil/PosixThread.h"
 #include "libutil/Configuration.h"
 
 #include <errno.h>
@@ -44,19 +48,32 @@
 #include <iostream>
 #include <iomanip>
 
+// Permit linking against older libraw1394 which didn't include this
+// function.
+#ifdef __GNUC__
+  #ifdef __APPLE__
+  #define WEAK_ATTRIBUTE weak_import
+  #else
+  #define WEAK_ATTRIBUTE __weak__
+  #endif
+  int raw1394_read_cycle_timer_and_clock(raw1394handle_t handle,
+      u_int32_t *cycle_timer, u_int64_t *local_time, clockid_t clk_id)
+      __attribute__((WEAK_ATTRIBUTE));
+#endif
+
 using namespace std;
 
 IMPL_DEBUG_MODULE( Ieee1394Service, Ieee1394Service, DEBUG_LEVEL_NORMAL );
 
 Ieee1394Service::Ieee1394Service()
     : m_configuration( NULL )
+    , m_resetHelper( NULL )
+    , m_armHelperNormal( NULL )
+    , m_armHelperRealtime( NULL )
     , m_handle( 0 )
     , m_handle_lock( new Util::PosixMutex("SRVCHND") )
-    , m_resetHandle( 0 )
     , m_util_handle( 0 )
     , m_port( -1 )
-    , m_RHThread_lock( new Util::PosixMutex("SRVCRH") )
-    , m_threadRunning( false )
     , m_realtime ( false )
     , m_base_priority ( 0 )
     , m_pIsoManager( new IsoHandlerManager( *this ) )
@@ -78,13 +95,13 @@ Ieee1394Service::Ieee1394Service()
 
 Ieee1394Service::Ieee1394Service(bool rt, int prio)
     : m_configuration( NULL )
+    , m_resetHelper( NULL )
+    , m_armHelperNormal( NULL )
+    , m_armHelperRealtime( NULL )
     , m_handle( 0 )
     , m_handle_lock( new Util::PosixMutex("SRVCHND") )
-    , m_resetHandle( 0 )
     , m_util_handle( 0 )
     , m_port( -1 )
-    , m_RHThread_lock( new Util::PosixMutex("SRVCRH") )
-    , m_threadRunning( false )
     , m_realtime ( rt )
     , m_base_priority ( prio )
     , m_pIsoManager( new IsoHandlerManager( *this, rt, prio ) )
@@ -110,7 +127,26 @@ Ieee1394Service::~Ieee1394Service()
 {
     delete m_pIsoManager;
     delete m_pCTRHelper;
-    stopRHThread();
+
+    m_resetHelper->Stop();
+    m_armHelperNormal->Stop();
+    m_armHelperRealtime->Stop();
+
+    for ( arm_handler_vec_t::iterator it = m_armHandlers.begin();
+          it != m_armHandlers.end();
+          ++it )
+    {
+        debugOutput(DEBUG_LEVEL_VERBOSE, "Unregistering ARM handler for 0x%016"PRIX64"\n", (*it)->getStart());
+        if(m_armHelperNormal) {
+            int err = raw1394_arm_unregister(m_armHelperNormal->get1394Handle(), (*it)->getStart());
+            if (err) {
+                debugError(" Failed to unregister ARM handler for 0x%016"PRIX64"\n", (*it)->getStart());
+                debugError(" Error: %s\n", strerror(errno));
+            }
+        } else {
+            debugWarning("ARM handler registered without valid ARM helper thread\n");
+        }
+    }
 
     delete m_pWatchdog;
     if ( m_handle ) {
@@ -118,10 +154,10 @@ Ieee1394Service::~Ieee1394Service()
     }
     delete m_handle_lock;
 
-    if ( m_resetHandle ) {
-        raw1394_destroy_handle( m_resetHandle );
-    }
-    delete m_RHThread_lock;
+    if(m_resetHelper) delete m_resetHelper;
+    if(m_armHelperNormal) delete m_armHelperNormal;
+    if(m_armHelperRealtime) delete m_armHelperRealtime;
+
     if ( m_util_handle ) {
         raw1394_destroy_handle( m_util_handle );
     }
@@ -215,6 +251,7 @@ Ieee1394Service::initialize( int port )
     if (port + 1 > nb_ports) {
         debugFatal("Requested port (%d) out of range (# ports: %d)\n", port, nb_ports);
     }
+    m_port = port;
 
     if(!m_pWatchdog) {
         debugError("No valid RT watchdog found.\n");
@@ -237,18 +274,47 @@ Ieee1394Service::initialize( int port )
         return false;
     }
 
-    m_resetHandle = raw1394_new_handle_on_port( port );
-    if ( !m_resetHandle ) {
-        if ( !errno ) {
-            debugFatal("libraw1394 not compatible\n");
-        } else {
-            debugFatal("Ieee1394Service::initialize: Could not get 1394 handle: %s",
-                strerror(errno) );
-            debugFatal("Is ieee1394 and raw1394 driver loaded?\n");
-        }
+    // helper threads for all sorts of ASYNC events
+    // note: m_port has to be set!
+    m_resetHelper = new HelperThread(*this, "BUSRST");
+    if ( !m_resetHelper ) {
+        debugFatal("Could not allocate busreset handler helper\n");
+        return false;
+    }
+    m_armHelperNormal = new HelperThread(*this, "ARMSTD");
+    if ( !m_armHelperNormal ) {
+        debugFatal("Could not allocate standard ARM handler helper\n");
+        return false;
+    }
+    m_armHelperRealtime = new HelperThread(*this, "ARMRT", m_realtime, m_base_priority);
+    if ( !m_armHelperRealtime ) {
+        debugFatal("Could not allocate realtime ARM handler helper\n");
         return false;
     }
 
+    // start helper threads
+    if(!m_resetHelper->Start()) {
+        debugFatal("Could not start busreset helper thread\n");
+        return false;
+    }
+    if(!m_armHelperNormal->Start()) {
+        debugFatal("Could not start standard ARM helper thread\n");
+        return false;
+    }
+    if(!m_armHelperRealtime->Start()) {
+        debugFatal("Could not start realtime ARM helper thread\n");
+        return false;
+    }
+
+    // attach the reset and ARM handlers
+    // NOTE: the handlers have to be started first, or there is no 1394handle
+    raw1394_set_bus_reset_handler( m_resetHelper->get1394Handle(),
+                                   this->resetHandlerLowLevel );
+
+    m_default_arm_handler = raw1394_set_arm_tag_handler( m_armHelperNormal->get1394Handle(),
+                                   this->armHandlerLowLevel );
+
+    // utility handle (used to read the CTR register)
     m_util_handle = raw1394_new_handle_on_port( port );
     if ( !m_util_handle ) {
         if ( !errno ) {
@@ -265,7 +331,8 @@ Ieee1394Service::initialize( int port )
     int err;
     uint32_t cycle_timer;
     uint64_t local_time;
-    err=raw1394_read_cycle_timer(m_handle, &cycle_timer, &local_time);
+    m_have_read_ctr_and_clock = false;
+    err = raw1394_read_cycle_timer(m_util_handle, &cycle_timer, &local_time);
     if(err) {
         debugOutput(DEBUG_LEVEL_VERBOSE, "raw1394_read_cycle_timer failed.\n");
         debugOutput(DEBUG_LEVEL_VERBOSE, " Error descr: %s\n", strerror(err));
@@ -277,11 +344,30 @@ Ieee1394Service::initialize( int port )
         debugWarning("==================================================================\n");
         m_have_new_ctr_read = false;
     } else {
-        debugOutput(DEBUG_LEVEL_VERBOSE, "This system supports the raw1394_read_cycle_timer call, using it.\n");
         m_have_new_ctr_read = true;
-    }
 
-    m_port = port;
+        // Only if raw1394_read_cycle_timer() is present is it worth even
+        // considering the option that raw1394_read_cycle_timer_and_clock()
+        // might be available.
+        if (raw1394_read_cycle_timer_and_clock != NULL) {
+            err = raw1394_read_cycle_timer_and_clock(m_util_handle, &cycle_timer, &local_time, CLOCK_MONOTONIC);
+            if (!err && Util::SystemTimeSource::setSource(CLOCK_MONOTONIC)==true)
+                m_have_read_ctr_and_clock = true;
+        }
+
+        if (m_have_read_ctr_and_clock) {
+            debugOutput(DEBUG_LEVEL_VERBOSE, "This system supports the raw1394_read_cycle_timer_and_clock call and the\n");
+            debugOutput(DEBUG_LEVEL_VERBOSE, "CLOCK_MONOTONIC clock source; using them.\n");
+        } else {
+            debugOutput(DEBUG_LEVEL_VERBOSE, "This system supports the raw1394_read_cycle_timer call, using it.\n");
+            debugOutput(DEBUG_LEVEL_NORMAL, "The raw1394_read_cycle_timer_and_clock call and/or the CLOCK_MONOTONIC\n");
+            debugOutput(DEBUG_LEVEL_NORMAL, "clock source is not available.\n");
+            debugOutput(DEBUG_LEVEL_NORMAL, "Fallback to raw1394_read_cycle_timer.\n");
+            debugOutput(DEBUG_LEVEL_NORMAL, "FFADO may be susceptible to NTP-induced clock discontinuities.\n");
+            debugOutput(DEBUG_LEVEL_NORMAL, "If this is an issue, upgrade libraw1394 to version 2.1.0 or later and/or\n");
+            debugOutput(DEBUG_LEVEL_NORMAL, "kernel 2.6.36 or later.\n");
+        }
+    }
 
     // obtain port name
     raw1394handle_t tmp_handle = raw1394_new_handle();
@@ -309,11 +395,9 @@ Ieee1394Service::initialize( int port )
 
     // set userdata
     raw1394_set_userdata( m_handle, this );
-    raw1394_set_userdata( m_resetHandle, this );
     raw1394_set_userdata( m_util_handle, this );
-    raw1394_set_bus_reset_handler( m_resetHandle,
-                                   this->resetHandlerLowLevel );
 
+    // increase the split-transaction timeout if required (e.g. for bebob's)
     int split_timeout = IEEE1394SERVICE_MIN_SPLIT_TIMEOUT_USECS;
     if(m_configuration) {
         m_configuration->getValueForSetting("ieee1394.min_split_timeout_usecs", split_timeout);
@@ -325,11 +409,11 @@ Ieee1394Service::initialize( int port )
     debugOutput(DEBUG_LEVEL_VERBOSE, "Minimum SPLIT_TIMEOUT: %d. Current: %d\n", split_timeout, timeout);
     if (timeout < split_timeout) {
         if(!setSplitTimeoutUsecs(getLocalNodeId(), split_timeout+124)) {
-            debugWarning("Could not set SPLIT_TIMEOUT to min requested (%d)\n", split_timeout);
+            debugOutput(DEBUG_LEVEL_VERBOSE, "Could not set SPLIT_TIMEOUT to min requested (%d)\n", split_timeout);
         }
         timeout = getSplitTimeoutUsecs(getLocalNodeId());
         if (timeout < split_timeout) {
-            debugWarning("Set SPLIT_TIMEOUT to min requested (%d) did not succeed\n", split_timeout);
+            debugOutput(DEBUG_LEVEL_VERBOSE, "Set SPLIT_TIMEOUT to min requested (%d) did not succeed\n", split_timeout);
         }
     }
 
@@ -355,8 +439,6 @@ Ieee1394Service::initialize( int port )
         return false;
     }
 
-    startRHThread();
-
     // make sure that the thread parameters of all our helper threads are OK
     if(!setThreadParameters(m_realtime, m_base_priority)) {
         debugFatal("Could not set thread parameters\n");
@@ -376,14 +458,17 @@ Ieee1394Service::setThreadParameters(bool rt, int priority) {
         debugOutput(DEBUG_LEVEL_VERBOSE, "Switching IsoManager to (rt=%d, prio=%d)\n",
                                          rt, priority);
         result &= m_pIsoManager->setThreadParameters(rt, priority);
-    }
+    } //else debugError("Bogus isomanager\n");
     if (m_pCTRHelper) {
         debugOutput(DEBUG_LEVEL_VERBOSE, "Switching CycleTimerHelper to (rt=%d, prio=%d)\n", 
                                          rt && IEEE1394SERVICE_CYCLETIMER_HELPER_RUN_REALTIME,
                                          IEEE1394SERVICE_CYCLETIMER_HELPER_PRIO);
         result &= m_pCTRHelper->setThreadParameters(rt && IEEE1394SERVICE_CYCLETIMER_HELPER_RUN_REALTIME,
                                                     IEEE1394SERVICE_CYCLETIMER_HELPER_PRIO);
-    }
+    } //else debugError("Bogus CTR helper\n");
+    if(m_armHelperRealtime) {
+        m_armHelperRealtime->setThreadParameters(rt, priority);
+    } //else debugError("Bogus RT ARM helper\n");
     return result;
 }
 
@@ -454,11 +539,21 @@ Ieee1394Service::getSystemTimeForCycleTimer(uint32_t ctr) {
 bool
 Ieee1394Service::readCycleTimerReg(uint32_t *cycle_timer, uint64_t *local_time)
 {
+    if (m_have_read_ctr_and_clock) {
+        int err;
+        err = raw1394_read_cycle_timer_and_clock(m_util_handle, cycle_timer, local_time, 
+                  Util::SystemTimeSource::getSource());
+        if(err) {
+            debugWarning("raw1394_read_cycle_timer_and_clock error: %s\n", strerror(errno));
+            return false;
+        }
+        return true;
+    } else
     if(m_have_new_ctr_read) {
         int err;
         err = raw1394_read_cycle_timer(m_util_handle, cycle_timer, local_time);
         if(err) {
-            debugWarning("raw1394_read_cycle_timer: %s\n", strerror(err));
+            debugWarning("raw1394_read_cycle_timer error: %s\n", strerror(errno));
             return false;
         }
         return true;
@@ -516,7 +611,7 @@ Ieee1394Service::readNoLock( fb_nodeid_t nodeId,
         return true;
     } else {
         #ifdef DEBUG
-        debugOutput(DEBUG_LEVEL_NORMAL,
+        debugOutput(DEBUG_LEVEL_VERBOSE,
                     "raw1394_read failed: node 0x%hX, addr = 0x%016"PRIX64", length = %zd\n",
                     nodeId, addr, length);
         #endif
@@ -650,7 +745,7 @@ Ieee1394Service::transactionBlock( fb_nodeid_t nodeId,
     // FIXME: simplify semantics
     if (nodeId == INVALID_NODE_ID) {
         debugWarning("operation on invalid node\n");
-        return false;
+        return NULL;
     }
     // NOTE: this expects a call to transactionBlockClose to unlock
     m_handle_lock->Lock();
@@ -910,8 +1005,10 @@ int
 Ieee1394Service::getSplitTimeoutUsecs(fb_nodeid_t nodeId)
 {
     Util::MutexLockHelper lock(*m_handle_lock);
-    quadlet_t split_timeout_hi;
-    quadlet_t split_timeout_low;
+
+    // Keep Valgrind quiet by including explicit assignment
+    quadlet_t split_timeout_hi = 0;
+    quadlet_t split_timeout_low = 0;
 
     debugOutput(DEBUG_LEVEL_VERBOSE, "reading SPLIT_TIMEOUT on node 0x%X...\n", nodeId);
 
@@ -983,9 +1080,15 @@ Ieee1394Service::resetHandlerLowLevel( raw1394handle_t handle,
                                        unsigned int generation )
 {
     raw1394_update_generation ( handle, generation );
-    Ieee1394Service* instance
-        = (Ieee1394Service*) raw1394_get_userdata( handle );
-    instance->resetHandler( generation );
+
+    Ieee1394Service::HelperThread *thread = reinterpret_cast<Ieee1394Service::HelperThread *>(raw1394_get_userdata( handle ));
+    if(thread == NULL) {
+        debugFatal("Bogus 1394 handle private data\n");
+        return -1;
+    }
+
+    Ieee1394Service& service = thread->get1394Service();
+    service.resetHandler( generation );
 
     return 0;
 }
@@ -1016,57 +1119,153 @@ Ieee1394Service::resetHandler( unsigned int generation )
     return true;
 }
 
-bool
-Ieee1394Service::startRHThread()
-{
-    int i;
+bool Ieee1394Service::registerARMHandler(ARMHandler *h) {
+    debugOutput(DEBUG_LEVEL_VERBOSE,
+                "Registering ARM handler (%p) for 0x%016"PRIX64", length %zu\n",
+                h, h->getStart(), h->getLength());
 
-    if ( m_threadRunning ) {
-        return true;
-    }
-    m_RHThread_lock->Lock();
-    i = pthread_create( &m_thread, 0, rHThread, this );
-    m_RHThread_lock->Unlock();
-    if (i) {
-        debugFatal("Could not start ieee1394 service thread\n");
+    // FIXME: note that this will result in the ARM handlers not running in a realtime context
+    int err = raw1394_arm_register(m_armHelperNormal->get1394Handle(), h->getStart(),
+                                   h->getLength(), h->getBuffer(), (octlet_t)h,
+                                   h->getAccessRights(),
+                                   h->getNotificationOptions(),
+                                   h->getClientTransactions());
+    if (err) {
+        debugError("Failed to register ARM handler for 0x%016"PRIX64"\n", h->getStart());
+        debugError(" Error: %s\n", strerror(errno));
         return false;
     }
-    m_threadRunning = true;
-
+    m_armHandlers.push_back( h );
     return true;
 }
 
-void
-Ieee1394Service::stopRHThread()
+bool Ieee1394Service::unregisterARMHandler( ARMHandler *h ) {
+    debugOutput(DEBUG_LEVEL_VERBOSE, "Unregistering ARM handler (%p) for 0x%016"PRIX64"\n",
+        h, h->getStart());
+
+    for ( arm_handler_vec_t::iterator it = m_armHandlers.begin();
+          it != m_armHandlers.end();
+          ++it )
+    {
+        if((*it) == h) {
+            int err = raw1394_arm_unregister(m_armHelperNormal->get1394Handle(), h->getStart());
+            if (err) {
+                debugError("Failed to unregister ARM handler (%p)\n", h);
+                debugError(" Error: %s\n", strerror(errno));
+            } else {
+                m_armHandlers.erase(it);
+                return true;
+            }
+        }
+    }
+    debugOutput(DEBUG_LEVEL_VERBOSE, " handler not found!\n");
+
+    return false;
+}
+/**
+ * @brief Tries to find a free ARM address range
+ *
+ * @param start  address to start with
+ * @param length length of the block needed (bytes)
+ * @param step   step to use when searching (bytes)
+ * @return The base address that is free, and 0xFFFFFFFFFFFFFFFF when failed
+ */
+nodeaddr_t Ieee1394Service::findFreeARMBlock( nodeaddr_t start, size_t length, size_t step ) {
+    debugOutput(DEBUG_LEVEL_VERBOSE,
+                "Finding free ARM block of %zd bytes, from 0x%016"PRIX64" in steps of %zd bytes\n",
+                length, start, step);
+
+    int cnt=0;
+    const int maxcnt=10;
+    int err=1;
+    Util::MutexLockHelper lock(*m_handle_lock);
+    while(err && cnt++ < maxcnt) {
+        // try to register
+        err = raw1394_arm_register(m_handle, start, length, 0, 0, 0, 0, 0);
+
+        if (err) {
+            debugOutput(DEBUG_LEVEL_VERBOSE, " -> cannot use 0x%016"PRIX64"\n", start);
+            debugError("    Error: %s\n", strerror(errno));
+            start += step;
+        } else {
+            debugOutput(DEBUG_LEVEL_VERBOSE, " -> use 0x%016"PRIX64"\n", start);
+            err = raw1394_arm_unregister(m_handle, start);
+            if (err) {
+                debugOutput(DEBUG_LEVEL_VERBOSE, " error unregistering test handler\n");
+                debugError("    Error: %s\n", strerror(errno));
+                return 0xFFFFFFFFFFFFFFFFLLU;
+            }
+            return start;
+        }
+    }
+    debugOutput(DEBUG_LEVEL_VERBOSE, " Could not find free block in %d tries\n",cnt);
+    return 0xFFFFFFFFFFFFFFFFLLU;
+}
+
+int
+Ieee1394Service::armHandlerLowLevel(raw1394handle_t handle,
+                                    unsigned long arm_tag,
+                                    byte_t request_type, unsigned int requested_length,
+                                    void *data)
 {
-    if ( m_threadRunning ) {
-        // wait for the thread to finish it's work
-        m_RHThread_lock->Lock();
-        pthread_cancel (m_thread);
-        pthread_join (m_thread, 0);
-        m_RHThread_lock->Unlock();
-        m_threadRunning = false;
+    Ieee1394Service::HelperThread *thread = reinterpret_cast<Ieee1394Service::HelperThread *>(raw1394_get_userdata( handle ));
+    if(thread == NULL) {
+        debugFatal("Bogus 1394 handle private data\n");
+        return -1;
+    }
+
+    Ieee1394Service& service = thread->get1394Service();
+    if(service.armHandler( arm_tag, request_type, requested_length, data )) {
+        return 0;
+    } else {
+        return -1;
     }
 }
 
-void*
-Ieee1394Service::rHThread( void* arg )
+bool
+Ieee1394Service::armHandler(  unsigned long arm_tag,
+                     byte_t request_type, unsigned int requested_length,
+                     void *data)
 {
-    Ieee1394Service* pIeee1394Service = (Ieee1394Service*) arg;
+    for ( arm_handler_vec_t::iterator it = m_armHandlers.begin();
+          it != m_armHandlers.end();
+          ++it )
+    {
+        if((*it) == (ARMHandler *)arm_tag) {
+            struct raw1394_arm_request_response *arm_req_resp;
+            arm_req_resp = (struct raw1394_arm_request_response *) data;
+            raw1394_arm_request_t arm_req = arm_req_resp->request;
+            raw1394_arm_response_t arm_resp = arm_req_resp->response;
 
-    while (true) {
-        // protect ourselves from dying
-        {
-            // use a scoped lock such that it is unlocked
-            // even if we are cancelled while running
-            // FIXME: check if this is true!
-//             Util::MutexLockHelper lock(*(pIeee1394Service->m_RHThread_lock));
-            raw1394_loop_iterate (pIeee1394Service->m_resetHandle);
+            debugOutput(DEBUG_LEVEL_VERBOSE,"ARM handler for address 0x%016"PRIX64" called\n",
+                (*it)->getStart());
+            debugOutput(DEBUG_LEVEL_VERBOSE," request type   : 0x%02X\n", request_type);
+            debugOutput(DEBUG_LEVEL_VERBOSE," request length : %04d\n", requested_length);
+
+            switch(request_type) {
+                case RAW1394_ARM_READ:
+                    (*it)->handleRead(arm_req);
+                    *arm_resp = *((*it)->getResponse());
+                    break;
+                case RAW1394_ARM_WRITE:
+                    (*it)->handleWrite(arm_req);
+                    *arm_resp = *((*it)->getResponse());
+                    break;
+                case RAW1394_ARM_LOCK:
+                    (*it)->handleLock(arm_req);
+                    *arm_resp = *((*it)->getResponse());
+                    break;
+                default:
+                    debugWarning("Unknown request type received, ignoring...\n");
+            }
+            return true;
         }
-        pthread_testcancel ();
     }
 
-    return 0;
+    debugOutput(DEBUG_LEVEL_VERBOSE,"default ARM handler called\n");
+
+    m_default_arm_handler(m_armHelperNormal->get1394Handle(), arm_tag, request_type, requested_length, data );
+    return true;
 }
 
 bool
@@ -1124,6 +1323,7 @@ signed int Ieee1394Service::allocateIsoChannelGeneric(unsigned int bandwidth) {
             break;
     }
     if (c < 63) {
+        debugOutput(DEBUG_LEVEL_VERBOSE, "found free iso channel %d\n", c);
         if (raw1394_bandwidth_modify(m_handle, bandwidth, RAW1394_MODIFY_ALLOC) < 0) {
             debugFatal("Could not allocate bandwidth of %d\n", bandwidth);
 
@@ -1151,6 +1351,59 @@ signed int Ieee1394Service::allocateIsoChannelGeneric(unsigned int bandwidth) {
     return -1;
 }
 
+/**
+ * Allocates a specific fixed iso channel for use by the interface.  Returns 
+ * -1 on error (due to the requested channel not being free) or the fixed iso
+ * channel number.
+ *
+ * Does not perform anything other than registering the channel and the
+ * bandwidth at the IRM
+ *
+ * Also allocates the necessary bandwidth (in ISO allocation units).
+ *
+ * FIXME: As in libiec61883, channel 63 is not requested; this is either a
+ * bug or it's omitted since that's the channel preferred by video devices.
+ *
+ * @chan the channel number being requested
+ * @param bandwidth the bandwidth to allocate for this channel
+ * @return the channel number
+ */
+signed int Ieee1394Service::allocateFixedIsoChannelGeneric(
+    unsigned int chan, unsigned int bandwidth
+    ) {
+    debugOutput(DEBUG_LEVEL_VERBOSE, "Allocating ISO channel %d using generic method...\n", chan );
+
+    Util::MutexLockHelper lock(*m_handle_lock);
+    struct ChannelInfo cinfo;
+
+    if (raw1394_channel_modify (m_handle, chan, RAW1394_MODIFY_ALLOC) == 0) {
+        if (raw1394_bandwidth_modify(m_handle, bandwidth, RAW1394_MODIFY_ALLOC) < 0) {
+            debugFatal("Could not allocate bandwidth of %d\n", bandwidth);
+
+            raw1394_channel_modify (m_handle, chan, RAW1394_MODIFY_FREE);
+            return -1;
+        } else {
+            cinfo.channel=chan;
+            cinfo.bandwidth=bandwidth;
+            cinfo.alloctype=AllocGeneric;
+
+            cinfo.xmit_node=-1;
+            cinfo.xmit_plug=-1;
+            cinfo.recv_node=-1;
+            cinfo.recv_plug=-1;
+
+            if (registerIsoChannel(chan, cinfo)) {
+                return chan;
+            } else {
+                raw1394_bandwidth_modify(m_handle, bandwidth, RAW1394_MODIFY_FREE);
+                raw1394_channel_modify (m_handle, chan, RAW1394_MODIFY_FREE);
+                return -1;
+            }
+        }
+    }
+    return -1;
+}
+
 /**
  * Allocates an iso channel for use by the interface in a similar way to
  * libiec61883.  Returns -1 on error (due to there being no free channels)
@@ -1389,3 +1642,99 @@ Ieee1394Service::show()
     #endif
     if (m_pIsoManager) m_pIsoManager->dumpInfo();
 }
+
+// the helper thread class
+Ieee1394Service::HelperThread::HelperThread(Ieee1394Service &parent, std::string name)
+: m_parent( parent )
+, m_name( name )
+, m_handle( NULL )
+, m_thread( *(new Util::PosixThread(this, name, false, 0, PTHREAD_CANCEL_DEFERRED)) )
+, m_iterate( false )
+, m_debugModule(parent.m_debugModule)
+{
+    m_handle = raw1394_new_handle_on_port( parent.m_port );
+    if(!m_handle) {
+        debugError("Could not allocate handle\n");
+        // FIXME: better error handling required
+    }
+    raw1394_set_userdata( m_handle, this );
+}
+
+Ieee1394Service::HelperThread::HelperThread(Ieee1394Service &parent, std::string name, bool rt, int prio)
+: m_parent( parent )
+, m_name( name )
+, m_handle( NULL )
+, m_thread( *(new Util::PosixThread(this, name, rt, prio, PTHREAD_CANCEL_DEFERRED)) )
+, m_iterate( false )
+, m_debugModule(parent.m_debugModule)
+{
+    m_handle = raw1394_new_handle_on_port( parent.m_port );
+    if(!m_handle) {
+        debugError("Could not allocate handle\n");
+        // FIXME: better error handling required
+    }
+    raw1394_set_userdata( m_handle, this );
+}
+
+Ieee1394Service::HelperThread::~HelperThread()
+{
+    m_thread.Stop();
+    delete &m_thread;
+    if(m_handle) {
+        raw1394_destroy_handle(m_handle);
+    }
+}
+
+bool
+Ieee1394Service::HelperThread::Init()
+{
+    m_iterate = true;
+    return true;
+}
+
+bool
+Ieee1394Service::HelperThread::Execute()
+{
+    if(m_iterate) {
+        int err;
+        err = raw1394_loop_iterate (m_handle);
+        if(err < 0) {
+            debugError("Failed to iterate handler\n");
+            return false;
+        } else {
+            return true;
+        }
+    } else {
+        Util::SystemTimeSource::SleepUsecRelative(1000);
+        return true;
+    }
+}
+
+void
+Ieee1394Service::HelperThread::setThreadParameters(bool rt, int priority)
+{
+    debugOutput( DEBUG_LEVEL_VERBOSE, "(%p) switch to: (rt=%d, prio=%d)...\n", this, rt, priority);
+    if (priority > THREAD_MAX_RTPRIO) priority = THREAD_MAX_RTPRIO; // cap the priority
+    if (rt) {
+        m_thread.AcquireRealTime(priority);
+    } else {
+        m_thread.DropRealTime();
+    }
+}
+
+bool
+Ieee1394Service::HelperThread::Start()
+{
+    return m_thread.Start() == 0;
+}
+
+bool
+Ieee1394Service::HelperThread::Stop()
+{
+    // request to stop iterating
+    m_iterate = false;
+    // poke the handler such that the iterate() returns
+    raw1394_wake_up(m_handle);
+    // stop the thread
+    return m_thread.Stop() == 0;
+}
