@@ -20,6 +20,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
+
 #include "config.h"
 
 #include "StreamProcessorManager.h"
@@ -58,6 +59,7 @@ StreamProcessorManager::StreamProcessorManager(DeviceManager &p)
     , m_shutdown_needed(false)
     , m_nbperiods(0)
     , m_WaitLock( new Util::PosixMutex("SPMWAIT") )
+    , m_max_diff_ticks( 50 ) 
 {
     addOption(Util::OptionContainer::Option("slaveMode",false));
     sem_init(&m_activity_semaphore, 0, 0);
@@ -83,6 +85,7 @@ StreamProcessorManager::StreamProcessorManager(DeviceManager &p, unsigned int pe
     , m_shutdown_needed(false)
     , m_nbperiods(0)
     , m_WaitLock( new Util::PosixMutex("SPMWAIT") )
+    , m_max_diff_ticks( 50 )
 {
     addOption(Util::OptionContainer::Option("slaveMode",false));
     sem_init(&m_activity_semaphore, 0, 0);
@@ -160,6 +163,10 @@ StreamProcessorManager::waitForActivity()
 
     if (m_activity_wait_timeout_nsec >= 0) {
 
+        // CLOCK_REALTIME must be used because that's what sem_timedwait() 
+        // uses.  This is safe - regardless of the clock used by
+        // Util::SystemTimeSource - so long as the resulting time is only
+        // used to implement a timeout in sem_timedwait().
         if (clock_gettime(CLOCK_REALTIME, &ts) == -1) {
             debugError("clock_gettime failed\n");
             return eAR_Error;
@@ -303,6 +310,94 @@ bool StreamProcessorManager::unregisterProcessor(StreamProcessor *processor)
     return false; //not found
 }
 
+bool StreamProcessorManager::streamingParamsOk(signed int period, signed int rate, signed int n_buffers)
+{
+    // Return true if the given parameter combination is valid.  If any
+    // parameter is set to -1 the currently set value is used in the test.
+    signed int min_period;
+
+    if (period < 0)
+        period = m_period;
+    if (rate < 0)
+        rate = m_nominal_framerate;
+    if (n_buffers < 0)
+        n_buffers = m_nb_buffers;
+
+    // For most interfaces data is transmitted with 8/16/32 samples per
+    // packet (at 1x, 2x and 4x rates respectively).  This more or less
+    // places a lower limit on the size of the period.  Furthermore, the
+    // current FFADO architecture dictates that m_nb_buffers can be no lower
+    // than 2.
+
+    if (n_buffers < 2) {
+        printMessage("FFADO requires at least 2 buffers\n");
+        return false;
+    }
+
+    // The boundary between 1x, 2x and 4x speed is taken from the RME driver
+    // since this seems to be the device with the widest available sampling
+    // rate range.
+    if (rate < 56000) {
+        // 1x speed
+        min_period = 8;
+    } else
+    if (rate < 112000) {
+        // 2x speed
+        min_period = 16;
+    } else {
+        // 4x speed
+        min_period = 32;
+    }
+
+    if (period < min_period) {
+        printMessage("At a rate of %d Hz, FFADO requires a buffer size of at least %d samples\n", 
+            rate, min_period);
+        return false;
+    }
+    return true;
+}
+
+void StreamProcessorManager::setPeriodSize(unsigned int period) {
+    // This method is called early in the initialisation sequence to set the
+    // initial period size.  However, at that point in time the stream
+    // processors haven't been registered so they won't have their buffers
+    // configured from here.  The initial allocation of the stream processor
+    // (SP) buffers happens from within the SP prepare() method.
+    //
+    // SP period size changes will normally only be acted on from here
+    // if the change comes about due to a runtime change in the buffer size,
+    // as happens via jack's setbufsize facility for example.
+
+    if (period == m_period)
+        return;
+
+    debugOutput( DEBUG_LEVEL_VERBOSE, "Setting period size to %d (was %d)\n", period, m_period);
+    m_period = period;
+
+    for ( StreamProcessorVectorIterator it = m_ReceiveProcessors.begin();
+          it != m_ReceiveProcessors.end();
+          ++it )
+    {
+        if ((*it)->periodSizeChanged(period) == false)
+            debugWarning("receive stream processor %p couldn't set period size\n", *it);
+    }
+    for ( StreamProcessorVectorIterator it = m_TransmitProcessors.begin();
+          it != m_TransmitProcessors.end();
+          ++it )
+    {
+        if ((*it)->periodSizeChanged(period) == false)
+            debugWarning("transmit stream processor %p couldn't set period size\n", *it);
+    }
+
+    // Keep the activity timeout in sync with the new period size.  See
+    // also comments about this in prepare().
+    if (m_nominal_framerate > 0) {
+        int timeout_usec = 2*1000LL * 1000LL * m_period / m_nominal_framerate;
+        debugOutput(DEBUG_LEVEL_VERBOSE, "setting activity timeout to %d\n", timeout_usec);
+        setActivityWaitTimeoutUsec(timeout_usec);
+    }
+}
+
 bool StreamProcessorManager::setSyncSource(StreamProcessor *s) {
     debugOutput( DEBUG_LEVEL_VERBOSE, "Setting sync source to (%p)\n", s);
     m_SyncSource = s;
@@ -375,7 +470,7 @@ bool StreamProcessorManager::prepare() {
     // if there are no stream processors registered,
     // fail
     if (m_ReceiveProcessors.size() + m_TransmitProcessors.size() == 0) {
-        debugFatal("No stream processors registered, can't do anything usefull\n");
+        debugFatal("No stream processors registered, can't do anything useful\n");
         return false;
     }
 
@@ -395,7 +490,6 @@ bool
 StreamProcessorManager::startDryRunning()
 {
     debugOutput( DEBUG_LEVEL_VERBOSE, "Putting StreamProcessor streams into dry-running state...\n");
-    debugOutput( DEBUG_LEVEL_VERBOSE, " Schedule start dry-running...\n");
     for ( StreamProcessorVectorIterator it = m_TransmitProcessors.begin();
             it != m_TransmitProcessors.end();
             ++it ) {
@@ -405,7 +499,7 @@ StreamProcessorManager::startDryRunning()
         }
         if (!(*it)->isDryRunning()) {
             if(!(*it)->scheduleStartDryRunning(-1)) {
-                debugError("Could not put SP %p into the dry-running state\n", *it);
+                debugError("Could not put '%s' SP %p into the dry-running state\n", (*it)->getTypeString(), *it);
                 return false;
             }
         } else {
@@ -421,7 +515,7 @@ StreamProcessorManager::startDryRunning()
         }
         if (!(*it)->isDryRunning()) {
             if(!(*it)->scheduleStartDryRunning(-1)) {
-                debugError("Could not put SP %p into the dry-running state\n", *it);
+                debugError("Could not put '%s' SP %p into the dry-running state\n", (*it)->getTypeString(), *it);
                 return false;
             }
         } else {
@@ -539,7 +633,7 @@ bool StreamProcessorManager::syncStartAll() {
         uint64_t ticks_at_period_margin = ticks_at_period + m_sync_delay;
         uint64_t pred_system_time_at_xfer = m_SyncSource->getParent().get1394Service().getSystemTimeForCycleTimerTicks(ticks_at_period_margin);
     
-        #ifdef DEBUG
+        #if DEBUG_EXTREME_ENABLE
         int64_t now = Util::SystemTimeSource::getCurrentTime();
         debugOutputExtreme(DEBUG_LEVEL_VERBOSE, "CTR  pred: %"PRId64", syncdelay: %"PRId64", diff: %"PRId64"\n", ticks_at_period, ticks_at_period_margin, ticks_at_period_margin-ticks_at_period );
         debugOutputExtreme(DEBUG_LEVEL_VERBOSE, "PREWAIT  pred: %"PRId64", now: %"PRId64", wait: %"PRId64"\n", pred_system_time_at_xfer, now, pred_system_time_at_xfer-now );
@@ -548,7 +642,7 @@ bool StreamProcessorManager::syncStartAll() {
         // wait until it's time to transfer
         Util::SystemTimeSource::SleepUsecAbsolute(pred_system_time_at_xfer);
     
-        #ifdef DEBUG
+        #if DEBUG_EXTREME_ENABLE
         now = Util::SystemTimeSource::getCurrentTime();
         debugOutputExtreme(DEBUG_LEVEL_VERBOSE, "POSTWAIT pred: %"PRId64", now: %"PRId64", excess: %"PRId64"\n", pred_system_time_at_xfer, now, now-pred_system_time_at_xfer );
         #endif
@@ -846,6 +940,8 @@ StreamProcessorManager::alignReceivedStreams()
         debugOutput( DEBUG_LEVEL_VERBOSE, " Average offsets:\n");
         int diff_between_streams_frames[nb_rcv_sp];
         aligned = true;
+
+        // first find whether the streams are aligned and what their offset is
         for ( i = 0; i < nb_rcv_sp; i++) {
             StreamProcessor *s = m_ReceiveProcessors.at(i);
 
@@ -855,13 +951,41 @@ StreamProcessorManager::alignReceivedStreams()
                 m_SyncSource, s, diff_between_streams[i], diff_between_streams_frames[i]);
 
             aligned &= (diff_between_streams_frames[i] == 0);
+        }
 
-            // reposition the stream
-            if(!s->shiftStream(diff_between_streams_frames[i])) {
-                debugError("Could not shift SP %p %d frames\n", s, diff_between_streams_frames[i]);
-                return false;
+        // if required, align the streams
+        int frames_to_shift_stream[nb_rcv_sp];
+        int min_shift = 9999;
+        if (!aligned) {
+            // find the minimum value (= earliest stream)
+            for ( i = 0; i < nb_rcv_sp; i++) {
+                if (diff_between_streams_frames[i] < min_shift) {
+                    min_shift = diff_between_streams_frames[i];
+                }
+            }
+            debugOutput( DEBUG_LEVEL_VERBOSE, " correcting shift with %d frames\n", min_shift);
+            // ensure that the streams are shifted only in the 'positive' direction
+            // i.e. that frames are only dropped, not added since that results
+            // in multiple writers for the data ringbuffer
+            // this also results in 'minimal shift' (not that it's required since the
+            // sync SP is part of the SP set)
+            for ( i = 0; i < nb_rcv_sp; i++) {
+                frames_to_shift_stream[i] = diff_between_streams_frames[i] - min_shift;
+                debugOutput(DEBUG_LEVEL_VERBOSE,
+                            "  going to drop %03d frames from stream %d\n",
+                            frames_to_shift_stream[i], i);
+            }
+            // perform the actual shift
+            for ( i = 0; i < nb_rcv_sp; i++) {
+                StreamProcessor *s = m_ReceiveProcessors.at(i);
+                // reposition the stream
+                if(!s->shiftStream(frames_to_shift_stream[i])) {
+                    debugError("Could not shift SP %p %d frames\n", s, frames_to_shift_stream[i]);
+                    return false;
+                }
             }
         }
+
         if (!aligned) {
             debugOutput(DEBUG_LEVEL_VERBOSE, "Streams not aligned, doing new round...\n");
         }
@@ -899,6 +1023,10 @@ bool StreamProcessorManager::start() {
     }
     if (!start_result) {
         debugFatal("Could not syncStartAll...\n");
+        // If unable to start, ensure stream processors and their handlers
+        // are in the stopped state, which is what the caller would reasonably
+        // expect if start() fails.
+        stop();
         return false;
     }
     debugOutput( DEBUG_LEVEL_VERBOSE, " Started...\n");
@@ -1099,7 +1227,7 @@ bool StreamProcessorManager::waitForPeriod() {
     uint64_t ticks_at_period_margin = ticks_at_period + m_sync_delay;
     uint64_t pred_system_time_at_xfer = m_SyncSource->getParent().get1394Service().getSystemTimeForCycleTimerTicks(ticks_at_period_margin);
 
-    #ifdef DEBUG
+    #if DEBUG_EXTREME_ENABLE
     int64_t now = Util::SystemTimeSource::getCurrentTime();
     debugOutputExtreme(DEBUG_LEVEL_VERBOSE, "CTR  pred: %"PRId64", syncdelay: %"PRId64", diff: %"PRId64"\n", ticks_at_period, ticks_at_period_margin, ticks_at_period_margin-ticks_at_period );
     debugOutputExtreme(DEBUG_LEVEL_VERBOSE, "PREWAIT  pred: %"PRId64", now: %"PRId64", wait: %"PRId64"\n", pred_system_time_at_xfer, now, pred_system_time_at_xfer-now );
@@ -1108,13 +1236,13 @@ bool StreamProcessorManager::waitForPeriod() {
     // wait until it's time to transfer
     Util::SystemTimeSource::SleepUsecAbsolute(pred_system_time_at_xfer);
 
-    #ifdef DEBUG
+    #if DEBUG_EXTREME_ENABLE
     now = Util::SystemTimeSource::getCurrentTime();
     debugOutputExtreme(DEBUG_LEVEL_VERBOSE, "POSTWAIT pred: %"PRId64", now: %"PRId64", excess: %"PRId64"\n", pred_system_time_at_xfer, now, now-pred_system_time_at_xfer );
     #endif
 
     // the period should be ready now
-    #ifdef DEBUG
+    #if DEBUG_EXTREME_ENABLE
     int rcv_fills[10];
     int xmt_fills[10];
     int i;
@@ -1239,7 +1367,7 @@ bool StreamProcessorManager::waitForPeriod() {
     // display message if the difference between two successive tick
     // values is more than 50 ticks. 1 sample at 48k is 512 ticks
     // so 50 ticks = 10%, which is a rather large jitter value.
-    if(diff-ticks_per_period > 50 || diff-ticks_per_period < -50) {
+    if(diff-ticks_per_period > m_max_diff_ticks || diff-ticks_per_period < -m_max_diff_ticks) {
         debugOutput(DEBUG_LEVEL_VERBOSE, "rather large TSP difference TS=%011"PRIu64" => TS=%011"PRIu64" (%d, nom %d)\n",
                                             m_time_of_transfer2, m_time_of_transfer, diff, ticks_per_period);
     }
@@ -1250,7 +1378,7 @@ bool StreamProcessorManager::waitForPeriod() {
                         "transfer period %d at %"PRIu64" ticks...\n",
                         m_nbperiods, m_time_of_transfer);
 
-    #ifdef DEBUG
+    #if DEBUG_EXTREME_ENABLE
     int rcv_bf=0, xmt_bf=0;
     for ( StreamProcessorVectorIterator it = m_ReceiveProcessors.begin();
         it != m_ReceiveProcessors.end();
@@ -1265,7 +1393,9 @@ bool StreamProcessorManager::waitForPeriod() {
     debugOutputExtreme( DEBUG_LEVEL_VERY_VERBOSE, 
                         "XF at %011"PRIu64" ticks, RBF=%d, XBF=%d, SUM=%d...\n",
                         m_time_of_transfer, rcv_bf, xmt_bf, rcv_bf+xmt_bf);
+    #endif
 
+    #ifdef DEBUG
     // check if xruns occurred on the Iso side.
     // also check if xruns will occur should we transfer() now
     for ( StreamProcessorVectorIterator it = m_ReceiveProcessors.begin();
