@@ -21,8 +21,6 @@
  *
  */
 
-#include "config.h"
-
 #include "StreamProcessor.h"
 #include "../StreamProcessorManager.h"
 
@@ -77,6 +75,8 @@ StreamProcessor::StreamProcessor(FFADODevice &parent, enum eProcessorType type)
     , m_ticks_per_frame( 0 )
     , m_dll_bandwidth_hz ( STREAMPROCESSOR_DLL_BW_HZ )
     , m_extra_buffer_frames( 0 )
+    , m_max_fs_diff_norm ( 0.01 )
+    , m_max_diff_ticks ( 50 )
     , m_in_xrun( false )
 {
     // create the timestamped buffer and register ourselves as its client
@@ -93,6 +93,51 @@ StreamProcessor::~StreamProcessor() {
     if (m_scratch_buffer) delete[] m_scratch_buffer;
 }
 
+bool
+StreamProcessor::periodSizeChanged(unsigned int new_periodsize) {
+    // This is called by the StreamProcessorManager whenever the period size
+    // is changed via setPeriodSize().  If the stream processor needs to do
+    // anything in response it can be done in this method.  Buffer size
+    // changes should only ever be made when streaming is not active.
+    //
+    // Return false if there was a problem dealing with the resize.
+    if (m_state!=ePS_Stopped && m_state!=ePS_Created) {
+        debugOutput(DEBUG_LEVEL_WARNING, "(%p) period change should only be done with streaming stopped\n", this);
+        return false;
+    }
+
+    // make the scratch buffer one period of frames long
+    m_scratch_buffer_size_bytes = new_periodsize * getEventsPerFrame() * getEventSize();
+    debugOutput( DEBUG_LEVEL_VERBOSE, " Allocate scratch buffer of %zd quadlets\n", m_scratch_buffer_size_bytes);
+    if(m_scratch_buffer) delete[] m_scratch_buffer;
+    m_scratch_buffer = new byte_t[m_scratch_buffer_size_bytes];
+    if(m_scratch_buffer == NULL) {
+        debugFatal("Could not allocate scratch buffer\n");
+        return false;
+    }
+
+    // set the parameters of ports we can:
+    // we want the audio ports to be period buffered,
+    // and the midi ports to be packet buffered
+    for ( PortVectorIterator it = m_Ports.begin();
+        it != m_Ports.end();
+        ++it )
+    {
+        debugOutput(DEBUG_LEVEL_VERBOSE, "Setting up port %s\n",(*it)->getName().c_str());
+        if(!(*it)->setBufferSize(m_StreamProcessorManager.getPeriodSize())) {
+            debugFatal("Could not set buffer size to %d\n",m_StreamProcessorManager.getPeriodSize());
+            return false;
+        }
+    }
+
+    if (!setupDataBuffer()) {
+        debugFatal("Could not setup data buffer\n");
+        return false;
+    }
+
+    return updateState();
+}
+
 bool
 StreamProcessor::handleBusResetDo()
 {
@@ -167,6 +212,48 @@ StreamProcessor::getNbPacketsIsoXmitBuffer()
 /***********************************************
  * Buffer management and manipulation          *
  ***********************************************/
+bool
+StreamProcessor::setupDataBuffer() {
+    assert(m_data_buffer);
+
+    unsigned int ringbuffer_size_frames = m_StreamProcessorManager.getNbBuffers() * m_StreamProcessorManager.getPeriodSize();
+    ringbuffer_size_frames += m_extra_buffer_frames;
+    ringbuffer_size_frames += 1; // to ensure that we can fit it all in there
+
+    bool result = true;
+
+    m_correct_last_timestamp = false;
+        
+    // initialize internal buffer
+    result &= m_data_buffer->setBufferSize(ringbuffer_size_frames);
+
+    result &= m_data_buffer->setEventSize( getEventSize() );
+    result &= m_data_buffer->setEventsPerFrame( getEventsPerFrame() );
+    if(getType() == ePT_Receive) {
+        result &= m_data_buffer->setUpdatePeriod( getNominalFramesPerPacket() );
+    } else {
+        result &= m_data_buffer->setUpdatePeriod( m_StreamProcessorManager.getPeriodSize() );
+    }
+    // Completing the buffer's setup and calling the prepare() method is not
+    // applicable unless the nominal rate (m_ticks_per_frame) has been
+    // configured.  This may not be the case when setupDataBuffer() is
+    // called from prepare() via periodSizeChanged(), but will be when called
+    // from doStop() and from periodSizeChanged() via the stream processor's
+    // setPeriodSize() method.
+    if (m_ticks_per_frame > 0) {
+        result &= m_data_buffer->setNominalRate(m_ticks_per_frame);
+        result &= m_data_buffer->setWrapValue(128L * TICKS_PER_SECOND);
+        result &= m_data_buffer->setBandwidth(STREAMPROCESSOR_DLL_FAST_BW_HZ / (double)TICKS_PER_SECOND);
+
+        result &= m_data_buffer->prepare(); // FIXME: the name
+
+        debugOutput(DEBUG_LEVEL_VERBOSE, "DLL info: nominal tpf: %f, update period: %d, bandwidth: %e 1/ticks (%e Hz)\n", 
+                    m_data_buffer->getNominalRate(), m_data_buffer->getUpdatePeriod(), m_data_buffer->getBandwidth(), m_data_buffer->getBandwidth() * TICKS_PER_SECOND);
+    }
+
+    return result;
+}
+
 void
 StreamProcessor::getBufferHeadTimestamp(ffado_timestamp_t *ts, signed int *fc)
 {
@@ -386,8 +473,9 @@ StreamProcessor::putPacket(unsigned char *data, unsigned int length,
             double fs_diff_norm = fs_diff/fs_nom;
             debugOutputExtreme(DEBUG_LEVEL_VERBOSE, "Nom fs: %12f, Instantanous fs: %12f, diff: %12f (%12f)\n",
                         fs_nom, fs_syt, fs_diff, fs_diff_norm);
-            if (fs_diff_norm > 0.01 || fs_diff_norm < -0.01) {
-                debugWarning( "Instantanous samplerate more than 1%% off nominal. [Nom fs: %12f, Instantanous fs: %12f, diff: %12f (%12f)]\n",
+            if (fs_diff_norm > m_max_fs_diff_norm || fs_diff_norm < -m_max_fs_diff_norm) {
+                debugWarning( "Instantanous samplerate more than %0.0f%% off nominal. [Nom fs: %12f, Instantanous fs: %12f, diff: %12f (%12f)]\n",
+                        m_max_fs_diff_norm*100,
                         fs_nom, fs_syt, fs_diff, fs_diff_norm);
             }
 
@@ -396,7 +484,7 @@ StreamProcessor::putPacket(unsigned char *data, unsigned int length,
                 // display message if the difference between two successive tick
                 // values is more than 50 ticks. 1 sample at 48k is 512 ticks
                 // so 50 ticks = 10%, which is a rather large jitter value.
-                if(diff-ticks_per_packet > 50 || diff-ticks_per_packet < -50) {
+                if(diff-ticks_per_packet > m_max_diff_ticks || diff-ticks_per_packet < -m_max_diff_ticks) {
                     debugOutput(DEBUG_LEVEL_VERBOSE,
                                 "cy %04d rather large TSP difference TS=%011"PRIu64" => TS=%011"PRIu64" (%d, nom %d)\n",
                                 (int)CYCLE_TIMER_GET_CYCLES(pkt_ctr), m_last_timestamp2,
@@ -704,8 +792,9 @@ StreamProcessor::getPacket(unsigned char *data, unsigned int *length,
                            fs_nom, fs_syt, fs_diff, fs_diff_norm);
 //                 debugOutput(DEBUG_LEVEL_VERBOSE, "Diff fs: %12f, m_last_timestamp: %011"PRIu64", m_last_timestamp2: %011"PRIu64"\n",
 //                            fs_diff, m_last_timestamp, m_last_timestamp2);
-                if (fs_diff_norm > 0.01 || fs_diff_norm < -0.01) {
-                    debugWarning( "Instantanous samplerate more than 1%% off nominal. [Nom fs: %12f, Instantanous fs: %12f, diff: %12f (%12f)]\n",
+                if (fs_diff_norm > m_max_fs_diff_norm || fs_diff_norm < -m_max_fs_diff_norm) {
+                    debugWarning( "Instantanous samplerate more than %0.0f%% off nominal. [Nom fs: %12f, Instantanous fs: %12f, diff: %12f (%12f)]\n",
+                           m_max_fs_diff_norm*100,
                            fs_nom, fs_syt, fs_diff, fs_diff_norm);
                 }
                 int ticks_per_packet = (int)(getTicksPerFrame() * getNominalFramesPerPacket());
@@ -713,7 +802,7 @@ StreamProcessor::getPacket(unsigned char *data, unsigned int *length,
                 // display message if the difference between two successive tick
                 // values is more than 50 ticks. 1 sample at 48k is 512 ticks
                 // so 50 ticks = 10%, which is a rather large jitter value.
-                if(diff-ticks_per_packet > 50 || diff-ticks_per_packet < -50) {
+                if(diff-ticks_per_packet > m_max_diff_ticks || diff-ticks_per_packet < -m_max_diff_ticks) {
                     debugOutput(DEBUG_LEVEL_VERBOSE,
                                 "cy %04d, rather large TSP difference TS=%011"PRIu64" => TS=%011"PRIu64" (%d, nom %d)\n",
                                 (int)CYCLE_TIMER_GET_CYCLES(pkt_ctr), m_last_timestamp2,
@@ -781,8 +870,8 @@ StreamProcessor::getPacket(unsigned char *data, unsigned int *length,
 send_empty_packet:
     // note that only the wait state changes are synchronized with the cycles
     if(m_state != m_next_state) {
-        debugOutput(DEBUG_LEVEL_VERBOSE, "Should update state from %s to %s\n",
-                                        ePSToString(m_state), ePSToString(m_next_state));
+        debugOutput(DEBUG_LEVEL_VERBOSE, "Should update '%s' state from %s to %s\n",
+                                        getTypeString(), ePSToString(m_state), ePSToString(m_next_state));
         // execute the requested change
         if (!updateState()) { // we are allowed to change the state directly
             debugError("Could not update state!\n");
@@ -798,6 +887,11 @@ send_empty_packet:
     return RAW1394_ISO_OK;
 }
 
+void StreamProcessor::packetsStopped() {
+    m_state = ePS_Stopped;
+    m_next_state = ePS_Stopped;
+}
+
 // Frame Transfer API
 /**
  * Transfer a block of frames from the event buffer to the port buffers
@@ -1067,29 +1161,9 @@ bool StreamProcessor::prepare()
 {
     debugOutput( DEBUG_LEVEL_VERBOSE, "Prepare SP (%p)...\n", this);
 
-    // make the scratch buffer one period of frames long
-    m_scratch_buffer_size_bytes = m_StreamProcessorManager.getPeriodSize() * getEventsPerFrame() * getEventSize();
-    debugOutput( DEBUG_LEVEL_VERBOSE, " Allocate scratch buffer of %zd quadlets\n", m_scratch_buffer_size_bytes);
-    if(m_scratch_buffer) delete[] m_scratch_buffer;
-    m_scratch_buffer = new byte_t[m_scratch_buffer_size_bytes];
-    if(m_scratch_buffer == NULL) {
-        debugFatal("Could not allocate scratch buffer\n");
+    if (periodSizeChanged(m_StreamProcessorManager.getPeriodSize()) == false)
         return false;
-    }
 
-    // set the parameters of ports we can:
-    // we want the audio ports to be period buffered,
-    // and the midi ports to be packet buffered
-    for ( PortVectorIterator it = m_Ports.begin();
-        it != m_Ports.end();
-        ++it )
-    {
-        debugOutput(DEBUG_LEVEL_VERBOSE, "Setting up port %s\n",(*it)->getName().c_str());
-        if(!(*it)->setBufferSize(m_StreamProcessorManager.getPeriodSize())) {
-            debugFatal("Could not set buffer size to %d\n",m_StreamProcessorManager.getPeriodSize());
-            return false;
-        }
-    }
     // the API specific settings of the ports should already be set,
     // as this is called from the processorManager->prepare()
     // so we can init the ports
@@ -1352,9 +1426,6 @@ StreamProcessor::doStop()
     assert(m_data_buffer);
 
     float ticks_per_frame;
-    unsigned int ringbuffer_size_frames = m_StreamProcessorManager.getNbBuffers() * m_StreamProcessorManager.getPeriodSize();
-    ringbuffer_size_frames += m_extra_buffer_frames;
-    ringbuffer_size_frames += 1; // to ensure that we can fit it all in there
 
     debugOutput(DEBUG_LEVEL_VERBOSE, "Enter from state: %s\n", ePSToString(m_state));
     bool result = true;
@@ -1369,23 +1440,7 @@ StreamProcessor::doStop()
         
             debugOutput(DEBUG_LEVEL_VERBOSE, "Initializing remote ticks/frame to %f\n", ticks_per_frame);
         
-            // initialize internal buffer
-            result &= m_data_buffer->setBufferSize(ringbuffer_size_frames);
-        
-            result &= m_data_buffer->setEventSize( getEventSize() );
-            result &= m_data_buffer->setEventsPerFrame( getEventsPerFrame() );
-            if(getType() == ePT_Receive) {
-                result &= m_data_buffer->setUpdatePeriod( getNominalFramesPerPacket() );
-            } else {
-                result &= m_data_buffer->setUpdatePeriod( m_StreamProcessorManager.getPeriodSize() );
-            }
-            result &= m_data_buffer->setNominalRate(ticks_per_frame);
-            result &= m_data_buffer->setWrapValue(128L * TICKS_PER_SECOND);
-            result &= m_data_buffer->setBandwidth(STREAMPROCESSOR_DLL_FAST_BW_HZ / (double)TICKS_PER_SECOND);
-            result &= m_data_buffer->prepare(); // FIXME: the name
-
-            debugOutput(DEBUG_LEVEL_VERBOSE, "DLL info: nominal tpf: %f, update period: %d, bandwidth: %e 1/ticks (%e Hz)\n", 
-                        m_data_buffer->getNominalRate(), m_data_buffer->getUpdatePeriod(), m_data_buffer->getBandwidth(), m_data_buffer->getBandwidth() * TICKS_PER_SECOND);
+            result &= setupDataBuffer();
             break;
         case ePS_DryRunning:
             if(!m_IsoHandlerManager.stopHandlerForStream(this)) {
@@ -1487,6 +1542,9 @@ StreamProcessor::doDryRunning()
         case ePS_WaitingForStreamDisable:
             result &= m_data_buffer->clearBuffer();
             m_data_buffer->setTransparent(true);
+            // If the stream has been running, clear the previous timestamps
+            // as they won't be relevant after a restart
+            m_last_timestamp = m_last_timestamp2 = 0;
             break;
         default:
             debugError("Entry from invalid state: %s\n", ePSToString(m_state));
@@ -1892,13 +1950,7 @@ StreamProcessor::dumpInfo()
     #ifdef DEBUG
     debugOutputShort( DEBUG_LEVEL_NORMAL, " StreamProcessor %p, %s:\n", this, ePTToString(m_processor_type));
     debugOutputShort( DEBUG_LEVEL_NORMAL, "  Port, Channel    : %d, %d\n", m_1394service.getPort(), m_channel);
-    IsoHandler *h = m_IsoHandlerManager.getHandlerForStream(this);
-    if (h) {
-        debugOutputShort( DEBUG_LEVEL_NORMAL, "  Packets, Dropped, Skipped : %d, %d, %d\n",
-                                              h->m_packets, h->m_dropped, h->m_skipped);
-    } else {
-        debugError("No handler for stream??\n");
-    }
+    m_IsoHandlerManager.dumpInfoForStream(this);
     uint64_t now = m_1394service.getCycleTimerTicks();
     debugOutputShort( DEBUG_LEVEL_NORMAL, "  Now                   : %011"PRIu64" (%03us %04uc %04ut)\n",
                         now,
