@@ -23,6 +23,7 @@
  */
 
 #include "config.h"
+
 #include "libutil/float_cast.h"
 
 #include "MotuTransmitStreamProcessor.h"
@@ -36,9 +37,20 @@
 
 #include "libutil/ByteSwap.h"
 
+#include "../../motu/motu_avdevice.h"
+
 #include <cstring>
 #include <assert.h>
 
+// Set to 1 to enable the generation of a 1 kHz test tone in analog output 1.  Even with
+// this defined to 1 the test tone will now only be produced if run with a non-zero 
+// debug level.
+#define TESTTONE 1
+
+#if TESTTONE
+#include <math.h>
+#endif
+
 /* Provide more intuitive access to GCC's branch predition built-ins */
 #define likely(x)   __builtin_expect((x),1)
 #define unlikely(x) __builtin_expect((x),0)
@@ -58,6 +70,7 @@ static inline uint32_t fullTicksToSph(int64_t timestamp) {
 MotuTransmitStreamProcessor::MotuTransmitStreamProcessor(FFADODevice &parent, unsigned int event_size )
         : StreamProcessor(parent, ePT_Transmit )
         , m_event_size( event_size )
+        , m_motu_model( 0 )
         , m_tx_dbc( 0 )
         , mb_head( 0 )
         , mb_tail( 0 )
@@ -71,12 +84,31 @@ MotuTransmitStreamProcessor::MotuTransmitStreamProcessor(FFADODevice &parent, un
    * will be dropped or corrupted in interesting ways.
    */
   midi_tx_period = lrintf(ceil((float)srate / 3125));
+
+  /* As for the receive case, the transmit code requires easy access to the
+   * MOTU model.
+   */
+  Motu::MotuDevice *dev = static_cast<Motu::MotuDevice *>(&parent);
+  m_motu_model = dev->m_motu_model;
+
+  /* The MOTU wants events to be sized in multiples of 4 bytes.  Because
+   * the channel data is 24 bit, this sometimes necessitates X pad bytes be 
+   * added where X is not a multiple of 3.  Store the number of such bytes
+   * for use later.
+   *
+   * The 828mk1 starts audio data at byte 4 while all other models start at 
+   * 10.  Since the difference between these is a multiple of 3 it doesn't
+   * matter which we use as the data origin.
+   */
+  m_event_pad_bytes = (event_size - 4) % 3;
 }
 
 unsigned int
 MotuTransmitStreamProcessor::getMaxPacketSize() {
     int framerate = m_Parent.getDeviceManager().getStreamProcessorManager().getNominalRate();
-    return framerate<=48000?616:(framerate<=96000?1032:1160);
+    // All devices tend to have larger rx packets than tx packets, so for
+    // now just use the rx numbers here.
+    return framerate<=48000?904:(framerate<=96000?1416:1672);
 }
 
 unsigned int
@@ -208,10 +240,8 @@ MotuTransmitStreamProcessor::generatePacketHeader (
             if(cycles_until_presentation >= MOTU_MIN_CYCLES_BEFORE_PRESENTATION)
             {
                 // we are not that late and can still try to transmit the packet
-                m_tx_dbc += fillDataPacketHeader((quadlet_t *)data, length, presentation_time);
+                fillDataPacketHeader((quadlet_t *)data, length, presentation_time);
                 m_last_timestamp = presentation_time;
-                if (m_tx_dbc > 0xff)
-                    m_tx_dbc -= 0x100;
                 return eCRV_Packet;
             }
             else   // definitely too late
@@ -222,10 +252,8 @@ MotuTransmitStreamProcessor::generatePacketHeader (
         else if(cycles_until_transmit <= MOTU_MAX_CYCLES_TO_TRANSMIT_EARLY)
         {
             // it's time send the packet
-            m_tx_dbc += fillDataPacketHeader((quadlet_t *)data, length, presentation_time);
+            fillDataPacketHeader((quadlet_t *)data, length, presentation_time);
             m_last_timestamp = presentation_time;
-            if (m_tx_dbc > 0xff)
-                m_tx_dbc -= 0x100;
             return eCRV_Packet;
         }
         else
@@ -271,11 +299,72 @@ MotuTransmitStreamProcessor::generatePacketData (
     if (m_data_buffer->readFrames(n_events, (char *)(data + 8))) {
         float ticks_per_frame = m_Parent.getDeviceManager().getStreamProcessorManager().getSyncSource().getTicksPerFrame();
 
+#if TESTTONE
+    /* Now things are beginning to stabilise, make things easier for others by only playing
+     * the test tone when run with a non-zero debug level.
+     */
+    if (getDebugLevel() > 0) {
+        // FIXME: remove this hacked in 1 kHz test signal to
+        // analog-1 when testing is complete.
+        signed int i, int_tpf = lrintf(ticks_per_frame);
+        unsigned char *sample = data+8+16;
+        for (i=0; i<n_events; i++, sample+=m_event_size) {
+            static signed int a_cx = 0;
+            // Each sample is 3 bytes with MSB in lowest address (ie: 
+            // network byte order).  After byte order swap, the 24-bit
+            // MSB is in the second byte of val.
+            signed int val = CondSwapToBus32(lrintf(0x7fffff*sin((1000.0*2.0*M_PI/24576000.0)*a_cx)));
+            memcpy(sample,((char *)&val)+1,3);
+            if ((a_cx+=int_tpf) >= 24576000) {
+                a_cx -= 24576000;
+            }
+        }
+    }
+#endif
+
+        // Set up each frames's SPH.
         for (int i=0; i < n_events; i++, quadlet += dbs) {
             int64_t ts_frame = addTicks(m_last_timestamp, (unsigned int)lrintf(i * ticks_per_frame));
             *quadlet = CondSwapToBus32(fullTicksToSph(ts_frame));
         }
 
+        /* Zero any pad bytes which, due to their number not being a multiple
+         * of 3, do not have a port associated with them.  Such bytes will
+         * never be written to in processWriteBlock() when the buffers are
+         * prepared, which means they could contain random data when put
+         * into the transmit packet unless explicitly zeroed here.
+         */
+        if (m_event_pad_bytes != 0) {
+            unsigned char *s = data+8+m_event_size-m_event_pad_bytes;
+            for (int i=0; i<n_events; i++, s+=m_event_size) {
+                *s = *(s+1) = 0;
+            }
+        }
+
+        /* To assist in verifying packet contents being sent to the
+         * interface, periodically print the data being sent when a debug
+         * mode is active.
+         */
+        if (getDebugLevel() > 0) {
+            static signed int pktcx = 0;
+            unsigned int i;
+            if (pktcx == 0) {
+                fprintf(stderr, "Packet to MOTU: length=%d, eventsize=%d, n_events=%d\n", *length, m_event_size, n_events);
+                for (i=0; i<*length; i++) {
+                    if ((i&0x000f) == 0)
+                        fprintf(stderr, "%08x  ", i);
+                    fprintf(stderr, "%02x ", data[i]);
+                    if ((i&0x000f) == 7)
+                        fprintf(stderr, "- ");
+                    if ((i&0x000f) == 0xf)
+                        fprintf(stderr, "\n");
+                }
+                fprintf(stderr, "\n");
+            }
+            if (++pktcx == 8000)
+                pktcx=0;
+        }
+
         return eCRV_OK;
     }
     else return eCRV_XRun;
@@ -298,7 +387,8 @@ MotuTransmitStreamProcessor::generateEmptyPacketHeader (
     *tag = 1;      // All MOTU packets have a CIP-like header
     *length = 8;
 
-    m_tx_dbc += fillNoDataPacketHeader ( (quadlet_t *)data, length );
+    fillNoDataPacketHeader ( (quadlet_t *)data, length );
+                                    
     return eCRV_OK;
 }
 
@@ -370,9 +460,7 @@ MotuTransmitStreamProcessor::generateSilentPacketHeader (
         if (cycles_until_presentation >= MOTU_MIN_CYCLES_BEFORE_PRESENTATION)
         {
             m_last_timestamp = presentation_time;
-            m_tx_dbc += fillDataPacketHeader((quadlet_t *)data, length, m_last_timestamp);
-            if (m_tx_dbc > 0xff)
-                m_tx_dbc -= 0x100;
+            fillDataPacketHeader((quadlet_t *)data, length, m_last_timestamp);
             return eCRV_Packet;
         }
         else
@@ -383,9 +471,7 @@ MotuTransmitStreamProcessor::generateSilentPacketHeader (
     else if (cycles_until_transmit <= MOTU_MAX_CYCLES_TO_TRANSMIT_EARLY)
     {
         m_last_timestamp = presentation_time;
-        m_tx_dbc += fillDataPacketHeader((quadlet_t *)data, length, m_last_timestamp);
-        if (m_tx_dbc > 0xff)
-            m_tx_dbc -= 0x100;
+        fillDataPacketHeader((quadlet_t *)data, length, m_last_timestamp);
         return eCRV_Packet;
     }
     else
@@ -440,14 +526,20 @@ unsigned int MotuTransmitStreamProcessor::fillDataPacketHeader (
     // all channels plus possibly other midi and control data.
     signed n_events = getNominalFramesPerPacket();
 
-    // construct the packet CIP-like header.  Even if this is a data-less
-    // packet the dbs field is still set as if there were data blocks
-    // present.  For data-less packets the dbc is the same as the previously
-    // transmitted block.
+    // For MOTU ISO packets containing data, the DBC is incremented before 
+    // being stored into the current packet's CIP-like header.
+    m_tx_dbc += n_events;
+    if (m_tx_dbc > 0xff)
+      m_tx_dbc -= 0x100;
+
+    // construct the packet CIP-like header.
     *quadlet = CondSwapToBus32(0x00000400 | ((m_Parent.get1394Service().getLocalNodeId()&0x3f)<<24) | m_tx_dbc | (dbs<<16));
     quadlet++;
     *quadlet = CondSwapToBus32(0x8222ffff);
     quadlet++;
+
+    // Return value is the increment applied to the DBC in case the caller
+    // needs to know this.
     return n_events;
 }
 
@@ -458,10 +550,9 @@ unsigned int MotuTransmitStreamProcessor::fillNoDataPacketHeader (
     // Size of a single data frame in quadlets.  See comment in
     // fillDataPacketHeader() regarding the Ultralite.
     unsigned dbs = m_event_size / 4;
-    // construct the packet CIP-like header.  Even if this is a data-less
-    // packet the dbs field is still set as if there were data blocks
-    // present.  For data-less packets the dbc is the same as the previously
-    // transmitted block.
+    // construct the packet CIP-like header.  Packets without data use
+    // the same DBC as the previous packet, so no DBC increment is needed
+    // here.
     *quadlet = CondSwapToBus32(0x00000400 | ((m_Parent.get1394Service().getLocalNodeId()&0x3f)<<24) | m_tx_dbc | (dbs<<16));
     quadlet++;
     *quadlet = CondSwapToBus32(0x8222ffff);
@@ -484,8 +575,14 @@ bool MotuTransmitStreamProcessor::processWriteBlock(char *data,
     bool no_problem=true;
     unsigned int i;
 
-    // Start with MIDI and control streams all zeroed.  Due to the sparce nature
-    // of these streams it is best to simply fill them in on an as-needs basis.
+    // Start with MIDI and control streams all zeroed.  Due to the sparce
+    // nature of these streams it is best to simply fill them in on an
+    // as-needs basis.  Note that the 828mk1 does not have MIDI ports or
+    // extra control data - audio data starts at offset 4 instead of 10. 
+    // However, apart from wasting a few cycles in the 828mk1 case no harm
+    // is done in zeroing these bytes - any audio channels in those
+    // positions will overwrite the zeros subsequently.  This saves the need
+    // for a conditional which for most MOTU interfaces would always be taken.
     for (i=0; i<nevents; i++) {
         memset(data+4+i*m_event_size, 0x00, 6);
     }
