@@ -237,14 +237,14 @@ Configuration::getValueForSetting(std::string path, int32_t &ref)
         Setting::Type t = s->getType();
         if(t == Setting::TypeInt) {
             ref = *s;
-            debugOutput(DEBUG_LEVEL_VERBOSE, "path '%s' has value %d\n", path.c_str(), ref);
+            debugOutput(DEBUG_LEVEL_VERY_VERBOSE, "path '%s' has value %d\n", path.c_str(), ref);
             return true;
         } else {
-            debugOutput(DEBUG_LEVEL_VERBOSE, "path '%s' has wrong type\n", path.c_str());
+            debugWarning("path '%s' has wrong type\n", path.c_str());
             return false;
         }
     } else {
-        debugOutput(DEBUG_LEVEL_VERBOSE, "path '%s' not found\n", path.c_str());
+        debugOutput(DEBUG_LEVEL_VERY_VERBOSE, "path '%s' not found\n", path.c_str());
         return false;
     }
 }
@@ -258,14 +258,14 @@ Configuration::getValueForSetting(std::string path, int64_t &ref)
         Setting::Type t = s->getType();
         if(t == Setting::TypeInt64) {
             ref = *s;
-            debugOutput(DEBUG_LEVEL_VERBOSE, "path '%s' has value %"PRId64"\n", path.c_str(), ref);
+            debugOutput(DEBUG_LEVEL_VERY_VERBOSE, "path '%s' has value %"PRId64"\n", path.c_str(), ref);
             return true;
         } else {
-            debugOutput(DEBUG_LEVEL_VERBOSE, "path '%s' has wrong type\n", path.c_str());
+            debugWarning("path '%s' has wrong type\n", path.c_str());
             return false;
         }
     } else {
-        debugOutput(DEBUG_LEVEL_VERBOSE, "path '%s' not found\n", path.c_str());
+        debugOutput(DEBUG_LEVEL_VERY_VERBOSE, "path '%s' not found\n", path.c_str());
         return false;
     }
 }
@@ -279,14 +279,14 @@ Configuration::getValueForSetting(std::string path, float &ref)
         Setting::Type t = s->getType();
         if(t == Setting::TypeFloat) {
             ref = *s;
-            debugOutput(DEBUG_LEVEL_VERBOSE, "path '%s' has value %f\n", path.c_str(), ref);
+            debugOutput(DEBUG_LEVEL_VERY_VERBOSE, "path '%s' has value %f\n", path.c_str(), ref);
             return true;
         } else {
-            debugOutput(DEBUG_LEVEL_VERBOSE, "path '%s' has wrong type\n", path.c_str());
+            debugWarning("path '%s' has wrong type\n", path.c_str());
             return false;
         }
     } else {
-        debugOutput(DEBUG_LEVEL_VERBOSE, "path '%s' not found\n", path.c_str());
+        debugOutput(DEBUG_LEVEL_VERY_VERBOSE, "path '%s' not found\n", path.c_str());
         return false;
     }
 }
@@ -303,7 +303,7 @@ Configuration::getSetting( std::string path )
             Setting &s = c->lookup(path);
             return &s;
         } catch (...) {
-            debugOutput(DEBUG_LEVEL_VERBOSE, "  %s has no setting %s\n",
+            debugOutput(DEBUG_LEVEL_VERY_VERBOSE, "  %s has no setting %s\n",
                         c->getName().c_str(), path.c_str());
         }
     }
@@ -333,7 +333,8 @@ Configuration::getValueForDeviceSetting(unsigned int vendor_id, unsigned model_i
     libconfig::Setting *s = getDeviceSetting( vendor_id, model_id );
     if(s) {
         try {
-            return s->lookupValue(setting, ref);
+            long long int refverylong = ref;
+            return s->lookupValue(setting, refverylong);
         } catch (...) {
             debugOutput(DEBUG_LEVEL_VERBOSE, "Setting %s not found\n", setting.c_str());
             return false;
@@ -422,7 +423,12 @@ Configuration::findDeviceVME( unsigned int vendor_id, unsigned model_id )
                 vme.vendor_name = tmp;
                 tmp = s["modelname"];
                 vme.model_name = tmp;
-                vme.driver = s["driver"];
+
+                if (!s.lookupValue("driver", vme.driver))
+                {
+                    std::string driver = s["driver"];
+                    vme.driver = convertDriver(driver);
+                }
                 return vme;
             } else {
                 debugError("BUG: vendor/model found but not found?\n");
@@ -485,6 +491,31 @@ Configuration::findFileName(std::string s) {
     return -1;
 }
 
+unsigned int
+Configuration::convertDriver(const std::string& driver) const {
+    if(driver == "BEBOB")
+        return eD_BeBoB;
+    if(driver == "FIREWORKS")
+        return eD_FireWorks;
+    if(driver == "GENERICAVC")
+        return eD_GenericAVC;
+    if(driver == "OXFORD")
+        return eD_Oxford;
+    if(driver == "MOTU")
+        return eD_MOTU;
+    if(driver == "DICE")
+        return eD_DICE;
+    if(driver == "METRICHALO")
+        return eD_MetricHalo;
+    if(driver == "RME")
+        return eD_RME;
+    if(driver == "BOUNCE")
+        return eD_Bounce;
+    if(driver == "DIGIDESIGN")
+        return eD_Digidesign;
+    return eD_Unknown; // Unknown
+}
+
 void
 Configuration::ConfigFile::readFile()
 {
