@@ -87,6 +87,7 @@ Device::Device( DeviceManager& d,
     , num_channels( 0 )
     , frames_per_packet( 0 )
     , speed800( 0 )
+    , provide_midi( 0 )
     , iso_tx_channel( -1 )
     , iso_rx_channel( -1 )
     , m_receiveProcessor( NULL )
@@ -356,6 +357,8 @@ bool
 Device::discover()
 {
     signed int i;
+    std::string id;
+
     unsigned int vendorId = getConfigRom().getNodeVendorId();
     // See note in Device::probe() about why we use the unit version here.
     unsigned int unitVersion = getConfigRom().getUnitVersion();
@@ -387,13 +390,18 @@ Device::discover()
       return false;
     }
 
+    id = std::string("dev0");
+    if (!getOption("id", id)) {
+        debugWarning("Could not retrieve id parameter, defaulting to 'dev0'\n");
+    }
+
     // Set up the shared data object for configuration data
-    i = rme_shm_open(&dev_config);
+    i = rme_shm_open(id, &dev_config);
     if (i == RSO_OPEN_CREATED) {
-        debugOutput( DEBUG_LEVEL_VERBOSE, "New configuration shared data object created\n");
+        debugOutput( DEBUG_LEVEL_VERBOSE, "New configuration shared data object created, ID %s\n", id.c_str());
     } else
     if (i == RSO_OPEN_ATTACHED) {
-        debugOutput( DEBUG_LEVEL_VERBOSE, "Attached to existing configuration shared data object\n");
+        debugOutput( DEBUG_LEVEL_VERBOSE, "Attached to existing configuration shared data object for ID %s\n", id.c_str());
     }
     if (dev_config == NULL) {
         debugOutput( DEBUG_LEVEL_WARNING, "Could not create/access shared configuration memory object, using process-local storage\n");
@@ -1125,7 +1133,7 @@ Device::readRegister(fb_nodeaddr_t reg) {
     
     quadlet = 0;
     if (get1394Service().read(0xffc0 | getNodeId(), reg, 1, &quadlet) <= 0) {
-        debugError("Error doing RME read from register 0x%06llx\n",reg);
+        debugError("Error doing RME read from register 0x%06"PRIx64"\n",reg);
     }
     return ByteSwapFromDevice32(quadlet);
 }
@@ -1136,7 +1144,7 @@ Device::readBlock(fb_nodeaddr_t reg, quadlet_t *buf, unsigned int n_quads) {
     unsigned int i;
 
     if (get1394Service().read(0xffc0 | getNodeId(), reg, n_quads, buf) <= 0) {
-        debugError("Error doing RME block read of %d quadlets from register 0x%06llx\n",
+        debugError("Error doing RME block read of %d quadlets from register 0x%06"PRIx64"\n",
             n_quads, reg);
         return -1;
     }
@@ -1154,7 +1162,7 @@ Device::writeRegister(fb_nodeaddr_t reg, quadlet_t data) {
     data = ByteSwapToDevice32(data);
     if (get1394Service().write(0xffc0 | getNodeId(), reg, 1, &data) <= 0) {
         err = 1;
-        debugError("Error doing RME write to register 0x%06llx\n",reg);
+        debugError("Error doing RME write to register 0x%06"PRIx64"\n",reg);
     }
 
     return (err==0)?0:-1;
@@ -1175,7 +1183,7 @@ Device::writeBlock(fb_nodeaddr_t reg, quadlet_t *data, unsigned int n_quads) {
     }
     if (get1394Service().write(0xffc0 | getNodeId(), reg, n_quads, data) <= 0) {
         err = 1;
-        debugError("Error doing RME block write of %d quadlets to register 0x%06llx\n",
+        debugError("Error doing RME block write of %d quadlets to register 0x%06"PRIx64"\n",
           n_quads, reg);
     }
 
