@@ -21,6 +21,8 @@
  *
  */
 
+#include "config.h"
+
 #include "downloader.h"
 
 #include "src/bebob/bebob_dl_mgr.h"
@@ -28,6 +30,7 @@
 
 #include "libieee1394/configrom.h"
 #include "libieee1394/ieee1394service.h"
+#include "version.h"
 
 #include <iostream>
 #include <cstdlib>
